PongComponent = class(function(o)

end)
function PongComponent:startup(object)
	-- get the velocity towards the hero character
	object.moves = true
	object.elasticity = 1
end
function PongComponent:update(object)
	-- make sure that we stay on screen
	-- perhaps I could add a warmup so that things can slide onto the screen properly... something to think about.
	if object.pos.x < 0 then
		object.pos.x = 0
		object.velocity.x = -object.velocity.x
	end
	if object.pos.x+object.w > bludG.camera.w then
		object.pos.x = bludG.camera.w-object.w
		object.velocity.x = -object.velocity.x
	end
	if object.pos.y < 0 then
		object.pos.y = 0
		object.velocity.y = -object.velocity.y
	end
	if object.pos.y > bludG.camera.h then
		object.pos.y = bludG.camera.h
		object.velocity.y = -object.velocity.y
	end
end
TowardsPlayerComponent = class(function(o)

end)
function TowardsPlayerComponent:startup(object)
	-- get the velocity towards the hero character
	object.moves = true
	local initialSpeed = object.dataFile.initialSpeed or 400
	object.velocity = mainState.substate.hero.pos:copy():sub(object.pos:copy()):normalize():mult(initialSpeed*r_scale)
end
-- moves the object in vertical waves as it moves across the screen
SineComponent = class(function(o)
end)
function SineComponent:startup(object)
	-- store the initial vertical position so that we can move it based on offsets from that
	self.starty = object.pos.y
	self.phase = object.dataFile.phase or 0
	self.amplitude = object.dataFile.amplitude or 20
end
function SineComponent:update(object)
	-- should eventually change the vertical amp to be in relative to the 7 v positions
	object.pos.y = self.starty + math.sin(object.syncCountFloat/8*math.pi*2+self.phase)*self.amplitude*r_scale
end
function timeInBeats(time)
	return player.currentBpm*(time/60)*4
end

-- moves the object in vertical waves as it moves across the screen
SyncMissileComponent = class(function(o)
end)
function SyncMissileComponent:startup(object)
	object.noSync = true
	self.startx = object.pos.x
end
function SyncMissileComponent:update(object)
	object.pos.x = Tweener.equations.inQuart(object.syncCountFloat/16, self.startx, -480, 1)
	-- print()
end

-- moves the object in vertical waves as it moves across the screen
OffsetShotComponent = class(function(o)
end)
function OffsetShotComponent:startup(object)
	object.noSync = true
	local oy = object.pos.y
	-- set the position to the center of the screen
	object.pos.y = bludG.camera.h/2
	-- get the ypos of the note, and the current ypos (note wise) of the hero, then offset (arrgh, this is fucking crazy!!)
	local speed = object.velocity:length()

	-- components only get loaded during play, so this is a safe assumption :D
	local heroPos = mainState.substate:screenHeightToNoteHeight(mainState.substate.hero.pos.y)
	heroPos = (heroPos+mainState.substate:screenHeightToNoteHeight(oy))%7
	local target = Vec2(mainState.substate.hero.pos.x, mainState.substate:noteHeightToScreenHeight(heroPos))
	target:sub(object.pos):normalize():mult(speed)
	object.velocity = target
end
function OffsetShotComponent:update(object)
	-- print()
end

-- moves the object in vertical waves as it moves across the screen
HomingShotComponent = class(function(o)
end)
function HomingShotComponent:startup(object)
	object.velocity:mult(1.1)
	object.deathin = 300
end
function HomingShotComponent:update(object)
	object.deathin = object.deathin-1
	if object.deathin<0 then
		object:kill()
	end
	-- get the current velocity and rotate it such that it points more at the hero
	local hero  = mainState.substate.hero
	local currentDir = object.velocity:angle()
	local speed = object.velocity:length()
	local targetAngle = hero.pos:copy():sub(object.pos):normalize():angle()
	if currentDir > targetAngle then
		object.velocity:normalize():rotate(0.01)
	else
		object.velocity:normalize():rotate(-0.01)
	end
	object.velocity:mult(speed)
end

-- moves the object in vertical waves as it moves across the screen
SplitComponent = class(function(o)
end)
function SplitComponent:startup(object)
	self.startSplit = object.dataFile.splitTime or 8
	self.splitNum = object.dataFile.splitNum or 2
	self.splitRot = object.dataFile.splitRot or 30
end
function SplitComponent:update(object)
	if object.syncCountFloat>self.startSplit and not self.split then
		self.split = true
		-- copy the object to a new synth object
		local rotStart = -self.splitRot/2
		for i=1,self.splitNum-1 do
			rotStart = rotStart + (self.splitRot/(self.splitNum-1))
			local newObj = SynthObject(object.pos.x, object.pos.y, object.dataFile)
			newObj.velocity.x, newObj.velocity.y = object.velocity.x, object.velocity.y
			-- rotate the original and the new one
			newObj.velocity:rotate(math.rad(rotStart))
			newObj.count = object.count
			newObj.levelInfo = object.levelInfo
			newObj.s = object.s
			newObj.moves = object.moves
			if newObj.animation then
				newObj.animation.startTime = object.animation.startTime
			end
			-- add the new one to the system
			mainState.substate.beats:add(newObj)
		end
		object.velocity:rotate(math.rad(-self.splitRot/2))
	end
end