PlayerModel = class(function(o)
	o:setupDirectories()
	o.songs = {}
	o.firstRun = true
	-- add a key subscription that reloads the existing world
	blud.key.pressedSubscribe(function(p, key)
		print(key)
		if key == 112 or key == 115 then
			o:loadWorldPack(o.currentWorld, true)
		end
	end,nil)
	o.serverSlots = 5
	o.highscores = {}
	o.globalHighscores = {}
	o.unlocks = {}
end)
function PlayerModel:save()
	local playerdata = self:saveTo({})
	persistence.store(blud.doc_root .. "/storage.lua", playerdata);
end
function PlayerModel:setBpm(bpm)
	pd:startList("fromOF")
	pd:addSymbol("bpm")
	pd:addFloat(bpm*4)
	pd:finish()
	self.currentBpm = bpm
end
function PlayerModel:startTimer()
	pd:startList("fromOF")
	pd:addSymbol("startTimer")
	pd:addFloat(100)
	pd:finish()
end
function PlayerModel:loadNumberOfServerLevels(callback)
	if self.numberOfServerLevels then return self.numberOfServerLevels end
	local lQuery = 	bludParseQuery("PlayerLevel")
	lQuery:whereKeyEquals("userId", gk:getId())
	lQuery:countObjects(function(count)
		if count then
			self.numberOfServerLevels = count
			if callback then
				callback(count)
			end
		end
	end)
end
function PlayerModel:setupDirectories()
	if not fs:isDirectory("/player_levels") then
		fs:mkDir("player_levels")
	end
end
function PlayerModel:loadWorldPack(packName, ignoreSprites, onComplete)
	local loadroutine = function()
		coroutine.yield()
		self.currentWorld = packName
		-- load in the data
		fs:dofile("/" .. packName .. "/SynthData.lua")
		-- load in the name gen as well
		fs:dofile("/" .. packName .. "/nameGen.lua")
		-- add the id based lookup to the synth data
		SynthData.getId = function(id)
			local obj
			for i,v in ipairs(SynthData) do
				if v.id == id then
					obj = v
				end
			end
			if not obj then
				obj = SynthData[id]
			end
			return obj
		end
		self.currentBpm = 110
		if type(SynthData.bpm) == 'number' then
			self.currentBpm = SynthData.bpm
		end
		coroutine.yield()
		self:loadSamples()
		coroutine.yield()
		self:loadLevelData()
		coroutine.yield()
		self:loadBackgroundData()
		coroutine.yield()
		onComplete()
		coroutine.yield()
		coroutine.yield()
		self.loadingOverlay:unzip()
		self.loadingOverlay.zg:unzip(function()
			mainState:removeState(self.loadingOverlay)	
		end)
	end
	self.loadingOverlay = LoadingOverlay(function()
		table.insert(runningCoroutines, coroutine.create(loadroutine))
	end)
	mainState:addState(self.loadingOverlay)	
end
function PlayerModel:saveTo(data)
	data.highscores = self.highscores
	data.globalHighscores = self.globalHighscores
	data.unlocks = self.unlocks
	data.firstRun = self.firstRun
	data.serverSlots = self.serverSlots
	return data
end
function PlayerModel:loadFrom(data)
	self.highscores = data.highscores
	if data.globalHighscores ~= nil then
		self.globalHighscores = data.globalHighscores
	end
	self.unlocks = data.unlocks
	if data.firstRun ~= nil then
		self.firstRun = data.firstRun
	end
	if data.serverSlots ~= nil then
		self.serverSlots = data.serverSlots
	end
end
function PlayerModel:storeHighscore(coincount, parseObject, songData)
	if parseObject then
		local index = parseObject:getId()
		if not self.globalHighscores[index] then
			self.globalHighscores[index] = {coincount, false}
		elseif coincount > self.globalHighscores[index][1] then
			self.globalHighscores[index] = {coincount, false}
		end
		return self.globalHighscores[index][1]
	elseif songData then
		local index = songData.worldPack..songData.name
		-- only going to store personal highscores for internal songs for now
		if not self.highscores[index] then
			self.highscores[index] = {coincount, false}
		elseif coincount > self.highscores[index][1] then
			self.highscores[index] = {coincount, false}
		end
		return self.highscores[index][1]
	end
	-- attempt updating server highscores for ones that have yet to be saved
end
function PlayerModel:updateServerHighscore()
	for k,v in pairs(self.highscores) do
		if not v[2] then
			WinScreenData.handleInternalScore(self, k, v[1], function(name, score)
				if score > v[1] then
					v[1] = score
				end
				v[2] = true
			end, true)
		end
	end
	for k,v in pairs(self.globalHighscores) do
		if not v[2] then
			WinScreenData.handleUserScore(self, k, v[1], function(name, score)
				if score > v[1] then
					v[1] = score
				end
				v[2] = true
			end, true)
		end
	end
end
function PlayerModel:getHighscore(parseObject, songData)
	-- should eventually return both the personal highscore and the global highscore if there is one
	if songData then
		local index = songData.worldPack..songData.name
		if self.highscores[index] then
			return self.highscores[index][1]
		end
	end
end
function PlayerModel:storeTris(coincount, parseObject, songData)
	if songData then
		local index = songData.worldPack..songData.name
		local lockInfo = {[1]='locked', [2]='locked', [3]='locked'}
		local percentageInfo = {}
		local targetInfo = {}
		if not self.unlocks[index] then
			self.unlocks[index] = {[1]='locked', [2]='locked', [3]='locked'}
		end
		if songData.stars then
			for i,v in ipairs(songData.stars) do
				percentageInfo[i] = coincount/v
				targetInfo[i] = v
				if v <= coincount and self.unlocks[index][i]=='locked' then
					lockInfo[i]='newUnlock'
					self.unlocks[index][i]='unlocked'
				else
					lockInfo[i] = self.unlocks[index][i]
				end
			end
		end
		return lockInfo, percentageInfo, targetInfo
	end
end
function PlayerModel:loadLevelData()
	if fs:exists("/" .. self.currentWorld .. "/levelData.lua") then
		fs:dofile("/" .. self.currentWorld .. "/levelData.lua")
	else
		levelData = {}
	end
end
function PlayerModel:loadBackgroundData()
	if fs:exists("/" .. self.currentWorld .. "/backgroundData.lua") then
		fs:dofile("/" .. self.currentWorld .. "/backgroundData.lua")
	else
		BackgroundData = nil
	end
end
function PlayerModel:loadSamples()
	self.samples = {}
	-- make a new coroutine, and submit it
	local loadCount = 0
	for i,v in ipairs(SynthData) do
		if v.sound then
			if type(v.sound) == 'string' and not self.samples[v.sound] then
				loadCount = self:loadSound(v.sound, loadCount)
			elseif type(v.sound) == 'table' then
				for counter,t in ipairs(v.sound) do
					if not self.samples[t[1]] then
						loadCount = self:loadSound(t[1], loadCount)
					end
				end
			end
		end
	end
end
function PlayerModel:loadSound(name, loadCount)
	pd:startList("fromOF")
	pd:addSymbol("soundfile")
	pd:addFloat(loadCount)
	pd:addSymbol(self.currentWorld .. "/sounds/" .. name)
	pd:finish()
	self.samples[name] = loadCount
	return loadCount+1
end
function PlayerModel:getSampleAndNote(synth, line)
	if not synth.sound then
		return nil
	end
	if type(synth.sound) == 'string' then
		return self.samples[synth.sound], SynthData.default_scale[line], synth.monoSynth
	elseif synth.sound[line][2] then
		return self.samples[synth.sound[line][1]], synth.sound[line][2], synth.sound[line][3]
	end
	return self.samples[synth.sound[line][1]], SynthData.default_scale[line], synth.sound[line][3]
end