NonBonusObj = class(Object, function(o, x, y, w, h)
  o.sprite = sprites['ui']['switch_1.png']
  Object.init(o, x, y, w, h)
  o.sprite = sprites['ui']['switch_1.png']
  o.sheet = sheets['ui']
  o.w = o.sprite:getTexWidth()
  o.h = o.sprite:getTexHeight()
  o:setPosition(x+o.sprite:getSpriteX(), y+o.sprite:getSpriteY())
  o.drawType = 'stretched'
  if r_scale == 4 then
    o.w = o.w*2
    o.h = o.h*2
    o:setPosition(x+o.sprite:getSpriteX()*2, y+o.sprite:getSpriteY()*2)
  end
end)

SynthObject = class(Object, function(o, x, y, dataFile)
	-- the width and height are based on the grid size
	o.selectorHeight = bludG.camera.h/9
	o.selectorWidth = bludG.camera.w/16
	Object.init(o, x, y, o.selectorWidth, o.selectorHeight)
	o.sheet = sheets['ui']
	o:setup(x, y, dataFile)
end)
function SynthObject:setup(x, y, dataFile)
	self:setPosition(x, y)
	local o = self
	dataFile = dataFile or {}
	o.dataFile = dataFile
	local self = o
	if dataFile.accel then
		self.acceleration = Vec2(dataFile.accel[1], dataFile.accel[2])
		if mainState.substate:is_a(SpaceHero3) then
			self.moves = true
		end
	end
	o.syncCount = 0
	o.syncCountFloat = 0
	o.no_sync = dataFile.no_sync
	o.no_ground_collide = dataFile.no_ground_collide
	if dataFile.img then
		if type(dataFile.img) == 'string' then
			self.animation = nil
			self.sprite = sprites['ui'][dataFile.img]
			self.drawType = 'stretched'
		elseif type(dataFile.img) == 'table' then
			-- load all of the images into an animation
			local images = {}
			for i,v in ipairs(dataFile.img) do
				if type(v) == "table" then
					for j=1,v[2] do
						table.insert(images, sprites['ui'][v[1]])	
					end
				else
					table.insert(images, sprites['ui'][v])
				end
			end
			self.animation = AnimationModel(images, {rate=41})
			self.animation:startAnimation()
			self.sprite = self.animation:getSprite()
		end
		-- pull the width and height from the first sprite
		-- self.w = self.sprite:getWidth()
		-- self.h = self.sprite:getHeight()
	end
	if SynthData.coin_tint and not dataFile.killer then
		local t = SynthData.coin_tint
		o:setColor(t[1], t[2], t[3])
	end
	o.scale = 1
	o.layer = 0
end
function SynthObject:setupComponents()
	local o = self
	-- add in all the components that we can find
	o.components = {}
	-- only do this on actual playback
	if mainState.substate:is_a(SpaceHero3) then
		if self.dataFile.components then
			for i,v in ipairs(self.dataFile.components) do
				if _G[v..'Component'] then
					local c = _G[v..'Component']()
					if c.startup then c:startup(o) end
					table.insert(o.components, c)
				end
			end
		end
	end
end
function SynthObject:recycle(x, y, dataFile)
  self:setColor(255, 255, 255, 255)
  self.alive = true
  self.exists = true
  self:setKey(false)
  self.moves = false
  self.velocity.x, self.velocity.y = 0,0
  self.acceleration.x, self.acceleration.y = 0,0
  self:setup(x, y, dataFile)
end
function SynthObject:setKey(setting)
	if setting then
		self.key = NonBonusObj(self.pos.x, self.pos.y, self.w, self.h)
		local t = SynthData.key_tint
		self:setColor(t[1], t[2], t[3])
	else
		self.key = nil
	end
end
function SynthObject:sync()
	self.syncCount = self.syncCount + 1
	self.syncCountFloat = self.syncCount
end
function timeInBeats(time)
  return player.currentBpm*(time/60)*4
end

function beatsToTime(beats)
  return beats*(1/(player.currentBpm/60))/4
end
function SynthObject:update()
	Object.update(self)
	if self.key then self.key:update() end
	-- attempt to get the approximate sync count
	-- the conversion from bpm to time: (player.currentBpm/60)*1/16+bludG.elapsed*2
	-- the conversion from time to beats: ???
	self.syncCountFloat = self.syncCountFloat+timeInBeats(bludG.elapsed)
	if self.animation then
		self.sprite = self.animation:getSprite()
	end
	if self.components then
		for i,v in ipairs(self.components) do
			if v.update then v:update(self) end
		end
	end
	-- for objects that are not sync'd we need to kill
	-- this is the bug where the missiles get nonsensical
	if self.pos.x < -self.w*2 then
		self:kill()
	end
end
function SynthObject:draw()
  -- draw debug
  local opx, opy = self.pos.x, self.pos.y
  self.pos.x = self.pos.x+self.w/2
  self.pos.y = self.pos.y+self.h/2
  if self.alive and self.sprite then
	if self.key then
		local t = SynthData.key_tint
		self:setColor(t[1], t[2], t[3])
	elseif not self.dataFile.killer then
		local t = SynthData.coin_tint
		self:setColor(t[1], t[2], t[3])
	end
    CenteredObject.draw(self)
  end
  self.pos.x, self.pos.y = opx, opy
end