-- includes code from
-- https://github.com/vrld/HardonCollider/blob/master/polygon.lua
--[[
Copyright (c) 2011 Matthias Richter

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Except as contained in this notice, the name(s) of the above copyright holders
shall not be used in advertising or otherwise to promote the sale, use or
other dealings in this Software without prior written authorization.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
]]--

TriangleObject = class(CenteredObject, function(o, x, y)
	CenteredObject.init(o, x, y, 10, 10)
	o.sprite = sprites['ui']['white.png']
	o.sheet = sheets['ui']
	o.size = 100
	o.rot = 0
	o.fingerDown = {}
	o.fingerCount = 0
	o:calculateCorners()
end)
local function onSameSide(a,b, c,d)
	local px, py = d.x-c.x, d.y-c.y
	local l = Vec2.det(px,py,  a.x-c.x, a.y-c.y)
	local m = Vec2.det(px,py,  b.x-c.x, b.y-c.y)
	return l*m >= 0
end
function TriangleObject:doesPointTouch(p)
	return onSameSide(p,self.tlc,self.trc,self.blc) and onSameSide(p,self.trc, self.tlc,self.blc) and onSameSide(p,self.blc, self.tlc,self.trc)
end
function TriangleObject:setRot(rot)
	self.rot = rot
	self:calculateCorners()
end
function TriangleObject:setSize(size)
	self.size = size
	self:calculateCorners()
end
function TriangleObject:calculateCorners()
	local corner = Vec2(0,-1):normalize():rotate(math.rad(self.rot))
	if not self.tl then
		self.tl, self.tr, self.bl = self.pos:copy(), self.pos:copy(), self.pos:copy()
		self.tlc, self.trc, self.blc = Vec2(0,0), Vec2(0,0), Vec2(0,0)
	else
		-- copy into the vectors
		self.tl:set(self.pos)
		self.tr:set(self.pos)
		self.bl:set(self.pos)
	end
	self.tl:add(corner:mult(self.size))
	self.tr:add(corner:normalize():rotate(math.rad(120)):mult(self.size))
	self.bl:add(corner:normalize():rotate(math.rad(120)):mult(self.size))
end
function TriangleObject:touchDown(x, y, id)
  if self:doesPointTouch(Vec2(x, y)) then
    self.fingerDown[id] = true
    self.fingerCount = self.fingerCount + 1
    if self.onTouch then 
      return self:onTouch()
    end
  end
  -- allow bubbling to continue
  return true
end
function TriangleObject:touchUp(x, y, id)
  -- don't actually commit the touch unless it starts and ends within the button
  if self:doesPointTouch(Vec2(x, y)) and self.fingerDown[id] then
    self.fingerDown[id] = nil
    self.fingerCount = self.fingerCount - 1
    if self.onPress then 
      local r_val = self:onPress()
      -- addTriSplash(Vec2(x, y))
      return r_val
    end
  end
  self.fingerDown[id] = nil
  return true
end
function TriangleObject:draw()
	-- print(self)
	local cameras = self.cameras or bludG.cameras
	if not self.cam then
		self.cam = Rectangle(0, 0, bludG.camera.w, bludG.camera.h)
	end
	local flip = 0
	if self.flip then flip = self.flip end

	if self.sprite then
		local camera = bludG.camera
			bludG.camera:worldToScreen(self.tl, self.tlc)
			bludG.camera:worldToScreen(self.tr, self.trc)
			bludG.camera:worldToScreen(self.bl, self.blc)
			local cam = Rectangle(0, 0, camera.w, camera.h)
			if self:doesRectangleTouch(camera) or self.forceDraw then
				-- calculate the position on the current camera
				self.sheet:addCornerTile(self.sprite, self.tlc.x, self.tlc.y, self.trc.x, self.trc.y, self.blc.x, self.blc.y, self.blc.x, self.blc.y, self.layer, flip, self.tint.r, self.tint.g, self.tint.b, self.tint.a)
			end
	end
end
-- mainState = WrapperState(Group());
-- mainState:changeState(TopLevelMenu())