function bgc(count)
	return {209, 207, 199}
end
--[[
	the data structure
	assumes a canvas at the top level
--]]
UICanvas = class(Object, function(o, canvasData)
	local l,t,w,h = canvasData.l or 0,canvasData.t or 0, canvasData.w or bludG.camera.w, canvasData.h or bludG.camera.h
	Object.init(o, l,t,w,h)
	o.canvasData = canvasData
	o.registeredObjs = {}
	o.pen = Vec2(o.pos.x,o.pos.y)
	if o.canvasData.paddingTop then
		o.pen.y = o.pen.y + o.canvasData.paddingTop
	end
	o.subs = Group()
	o.drawType = 'stretched'
	o:setColor(0,0,0,0)
	for i,v in ipairs(o.canvasData) do
		if _G[v.t] then
			if v.valign == 'bottom'then
				local sub = o.subs:add(_G[v.t](o.pen.x, bludG.camera.h-35*r_scale,v,o))		
			else
				if v.paddingTop then
					o.pen.y = o.pen.y+v.paddingTop
				end
				local sub = o.subs:add(_G[v.t](o.pen.x, o.pen.y,v,o))
				o.pen.y = o.pen.y+(sub.lh or sub.h)
				if v.paddingBottom then
					o.pen.y = o.pen.y+v.paddingBottom
				end
			end
		end
	end
	if o.canvasData.paddingBottom then
		o.pen.y = o.pen.y + o.canvasData.paddingBottom
	end
	o.h = o.pen.y
end)
function UICanvas:update()
	self.subs:update()
	Object.update(self)
end
function UICanvas:draw()
	self.subs:draw()
	Object.draw(self)
end
function UICanvas:touchDown(x, y, id) self.subs:touchDown(x, y, id) end
function UICanvas:touchMoved(x, y, id) self.subs:touchMoved(x, y, id) end
function UICanvas:touchUp(x, y, id) self.subs:touchUp(x, y, id) end
function UICanvas:registerID(id, obj)
	self.registeredObjs[id]=obj
end
function UICanvas:getObject(id)
	return self.registeredObjs[id]
end
UIline = class(Object, function(o, x, y, lineData, canvas)
	Object.init(o, x, y, bludG.camera.w, 35*r_scale)
	o.lh = o.h
	o.canvas = canvas
	o.subs = Group()
	if lineData.id then o.canvas:registerID(lineData.id, o) end
	local c = lineData.c or {209, 207, 199}
	o:setColor(c[1], c[2], c[3])
	o.sprite = sprites['ui']['white.png']
	o.drawType = 'stretched'
	o.penLeft = Vec2(x,y)
	o.penRight = Vec2(bludG.camera.w+6*r_scale,y)
	local subs = lineData.subs or {}
	for i,v in ipairs(subs) do
		o:addSub(v)
	end
end)
function UIline:addSub(data)
	local o = self
	local v = data
	if _G[v.t] then
		local pen = o.penLeft
		if v.align and v.align == 'right' then
			pen = o.penRight
		end
		local sub = o.subs:add(_G[v.t](pen.x, pen.y,v,o.canvas))
		if sub.h > self.lh then self.lh = sub.h end
		if pen == o.penRight then
			pen.x = sub.pos.x
			if sub.paddingLeft then
				pen.x = pen.x + sub.paddingLeft
			end
		else
			pen.x = sub.pos.x+sub.w
		end
	end
end
function UIline:update()
	self.subs:update()
	Object.update(self)
end
function UIline:draw()
	Object.draw(self)
	self.subs:draw()
end
function UIline:touchDown(x, y, id) self.subs:touchDown(x, y, id) end
function UIline:touchMoved(x, y, id) self.subs:touchMoved(x, y, id) end
function UIline:touchUp(x, y, id) self.subs:touchUp(x, y, id) end
UIspacer = class(TiltedRect, function(o, x, y, buttonData,canvas)
	TiltedRect.init(o, x, y, 5*r_scale, 35*r_scale)
	local c = buttonData.c or {getColor()}
	o:setColor(c[1], c[2], c[3])
	o.canvas = canvas
	o.sprite = sprites['ui']['white.png']
	if buttonData.align and buttonData.align == 'right' then
		o:setPosition(o.pos.x-o.w+6*r_scale, o.pos.y)
	end
end)
UIbutton = class(TiltedRect, function(o, x, y, buttonData, canvas)
	TiltedRect.init(o, x, y, 75*r_scale, 35*r_scale)
	o.canvas = canvas
	local c = buttonData.c or {147,146,140}
	o:setColor(c[1], c[2], c[3])
	o.sprite = sprites['ui']['white.png']
	if buttonData.align and buttonData.align == 'right' then
		o:setPosition(o.pos.x-o.w, o.pos.y)
	end
	if buttonData.id then o.canvas:registerID(buttonData.id, o) end
	-- add the overlay text
	if buttonData.text then
		o.text = StringObject(o.pos.x, o.pos.y, buttonData.text, nil, 0.5)
		o.text:setPosition(o.pos.x+o.w/2-o.text.w/2, o.pos.y+o.h/2-o.text.h/2)
	end
	if buttonData.onPress then
		o.onPress = function()
			buttonData.onPress(canvas)
		end
	end
	if buttonData.url and gk then
		o.onPress = function()
			gk:openUrl(buttonData.url)
		end		
	end
	o.touchDown = CameraButton.touchDown
	o.touchUp = CameraButton.touchUp
end)
function UIbutton:draw()
	TiltedRect.draw(self)
	if self.text then self.text:draw() end
end
UIsmallworld = class(Object, function(o, x, y, data)
	Object.init(o, x, y, 90*r_scale, 35*r_scale)
	-- setup the tri and the name
	if data.world then
		o.tri = TriangleObject(o.pos.x+15*r_scale, o.pos.y+o.h/2)
		o.tri:setSize(22*r_scale)
		o.tri:setRot(-19)
		local world = getWorldFromName(data.world)
		o.tri:setColor(world[2][1], world[2][2], world[2][3])
		o.text = StringObject(o.pos.x+30*r_scale, o.pos.y, data.world, nil, 0.5)
		o.text:setPosition(o.pos.x+10*r_scale, o.pos.y+o.h/2-o.text.h/2)
		o.text:setColor(0,0,0)
	end
end)
function UIsmallworld:draw()
	if self.tri then
		self.tri:draw()
		self.text:draw()
	end
end
UIbigworld = class(Object, function(o, x, y, data)
	Object.init(o, x, y, 90*r_scale, 35*r_scale)
	-- setup the tri and the name
	if data.world then
		o.tri = TriangleObject(o.pos.x+o.w-25*r_scale, o.pos.y+o.h/2)
		o.tri:setSize(50*r_scale)
		o.tri:setRot(-19)
		o.tri.forceDraw = true
		local world = data.world
		o.tri:setColor(world[2][1], world[2][2], world[2][3])
		local worldText = data.text or data.world[1]
		o.text = StringObject(o.pos.x+30*r_scale, o.pos.y, worldText, nil, 0.75)
		o.text:setPosition(o.pos.x+10*r_scale, o.pos.y+o.h/2-o.text.h/2)
		o.text:setColor(0,0,0)
	end
end)
function UIbigworld:draw()
	if self.tri then
		self.tri:draw()
		self.text:draw()
	end
end
UIlabel = class(Object, function(o, x, y, data, canvas)
	Object.init(o, x, y, 90*r_scale, 35*r_scale)
	-- setup the tri and the name
	o.canvas = canvas
	if data.id then o.canvas:registerID(data.id, o) end
	if data.text then
		o.text = StringObject(o.pos.x+30*r_scale, o.pos.y, data.text, nil, 0.5)
		o.text:setPosition(o.pos.x+10*r_scale, o.pos.y+o.h/2-o.text.h/2)
		o.text:setColor(0,0,0)
		o.w = o.text.w+30*r_scale
	end
end)
function UIlabel:draw()
	if self.text then
		self.text:draw()
	end
end
UIsmallStars = class(Object, function(o, x, y, data)
	Object.init(o, x, y, 90*r_scale, 35*r_scale)
	-- alignment
	if data.align and data.align == 'right' then
		o:setPosition(o.pos.x-o.w+6*r_scale, o.pos.y)
	end
	o.subs = Group()
	-- get the unlock info for the current level
	local ld = data.levelData
	local index = ld.worldPack..ld.name
	local lockInfo = {[1]='locked', [2]='locked', [3]='locked'}
	if ld.stars and player.unlocks[index] then
		for i,v in ipairs(ld.stars) do
			lockInfo[i] = player.unlocks[index][i]
		end
	end
	local triSize = 18*r_scale
	local padding = 90*r_scale
	local pen = Vec2(o.pos.x, o.pos.y+o.h/2)
	for i,v in ipairs(lockInfo) do
		-- add the background tri
		local t = o.subs:add(TriangleObject(pen.x, pen.y))
		t:setSize(triSize)
		t:setRot(-19)
		t:setColor(0,0,0)
		t.forceDraw = true
		-- if we need to, add an overlay
		if lockInfo[i]=='unlocked' then
			local t = o.subs:add(TriangleObject(pen.x-10*r_scale, pen.y-2*r_scale))
			t:setSize(triSize*0.9)
			t:setRot(-19)
			t:setColor(getYellow())
			t.forceDraw = true
		end
		pen.x = pen.x+triSize*1.5
	end
end)
function UIsmallStars:draw()
	self.subs:draw()
end
UIworldselect = class(Object, function(o, x, y, lineData, canvas)
	Object.init(o, x, y, bludG.camera.w, 80*r_scale)
	o.subs = Group()
	o.tris = Group()
	o:setColor(0,0,0,0)
	o.sprite = sprites['ui']['white.png']
	o.canvas = canvas
	local pen = Vec2(x+60*r_scale, y)
	for i,world in ipairs(WorldData) do
		o:addTri(pen, world, function(tri)
			-- -- change the size of all the tris
			fs:dofile("/" .. world[3] .. "/nameGen.lua")
			o.tris:call(function(v)
				if v ~= tri then
					if v.sizeTween then Tweener:removeTween(v.sizeTween) end
					v.sizeTween = Tweener:addTween(v, {tsize=30*r_scale}, {time=0.5})
				end
			end)
			if tri.sizeTween then Tweener:removeTween(tri.sizeTween) end			
			tri.sizeTween = Tweener:addTween(tri, {tsize=50*r_scale}, {func="outBounce", time=0.5})
			if lineData.onPress then
				lineData.onPress(o.canvas, world[3])
			end
		end)
	end
	o.tris.members[1].onPress()
	-- o.setup = true
	o.touchDown = UICanvas.touchDown
	o.touchMoved = UICanvas.touchMoved
	o.touchUp = UICanvas.touchUp
end)
function UIworldselect:addTri(fc, world, onPress)
	local o = self
	o.rotOff = -19
	o.tsize = 40*r_scale
	o.padding = 120*r_scale
	local t = o.subs:add(TriangleObject(fc.x, fc.y))
	o.tris:add(t)
	t:setSize(o.tsize)
	t:setRot(self.rotOff)
	t.tsize = o.tsize
	-- print(t.tsize)
	t:setColor(world[2][1], world[2][2], world[2][3])
	t.forceDraw = true
	t.st = o.subs:add(StringObject(fc.x-100*r_scale, fc.y-20*r_scale, world[1], nil, 0.5))
	t.st:setPosition(fc.x-t.st.w/2, fc.y-t.st.h/2)
	t.st:setColor(0,0,0)
	t.onPress = function()
		onPress(t)
	end
	fc.x = fc.x+100*r_scale
end
function UIworldselect:draw()
	self.subs:draw()
end
function UIworldselect:update()
	self.subs:update()
	self.tris:call(function(v)
		v:setSize(v.tsize)
	end)
end
UIBpmEditor = class(DragArea, function(o, x, y, data, canvas)
	DragArea.init(o, x, y, 80*r_scale, 35*r_scale)
	o.subs = Group()
	o.tris = Group()
	o:setColor(0,0,0,0)
	o.sprite = sprites['ui']['white.png']
	o.canvas = canvas

	o.sprite = sprites['ui']['white.png']
	o.drawType = 'stretched'
	o:setColor(0,0,0)
	local callback = function(value)
		data.callback(value, canvas)
	end
	o.fvalue = data.value or 120
	o.value = o.fvalue
	o.bpmText = o.subs:add(StringObject(0,0,o.value,nil,0.75))
	o.bpmText:setColor(0,0,0)
	o.bpmText:setPosition(o.pos.x+o.w/2-o.bpmText.w/2,o.pos.y+o.h/2-o.bpmText.h/2)
	o.bpmText:setLayer(1)
	o.onMove = function(da, x, y, id)
		local diff = 1/r_scale*(x-da.lastPos[id].x)*0.1
		o.fvalue = o.fvalue+diff
		o.value = math.floor(o.fvalue)
		if callback then callback(o.value) end
		o.bpmText:setValue(o.value)
		o.bpmText:setPosition(o.pos.x+o.w/2-o.bpmText.w/2,o.pos.y+o.h/2-o.bpmText.h/2)
	end
	o.ltri = o.subs:add(TriangleObject(x+10*r_scale, y+o.h/2))
	o.ltri:setSize(15*r_scale)
	o.ltri:setRot(-90)
	o.ltri:setColor(147,146,140)
	o.ltri.onPress = function()
		o.fvalue = o.fvalue-1
		o.value = math.floor(o.fvalue)
		if callback then callback(o.value) end
		o.bpmText:setValue(o.value)
		o.bpmText:setPosition(o.pos.x+o.w/2-o.bpmText.w/2,o.pos.y+o.h/2-o.bpmText.h/2)
	end
	o.rtri = o.subs:add(TriangleObject(x+o.w-10*r_scale, y+o.h/2))
	o.rtri:setSize(15*r_scale)
	o.rtri:setRot(90)
	o.rtri:setColor(147,146,140)
	o.rtri.onPress = function()
		o.fvalue = o.fvalue+1
		o.value = math.floor(o.fvalue)
		if callback then callback(o.value) end
		o.bpmText:setValue(o.value)
		o.bpmText:setPosition(o.pos.x+o.w/2-o.bpmText.w/2,o.pos.y+o.h/2-o.bpmText.h/2)
	end
end)
function UIBpmEditor:touchDown(x, y, id)
	self.ltri:touchDown(x, y, id)
	self.rtri:touchDown(x, y, id)
	DragArea.touchDown(self, x, y, id)	
end
function UIBpmEditor:touchUp(x, y, id)
	self.ltri:touchUp(x, y, id)
	self.rtri:touchUp(x, y, id)
	DragArea.touchUp(self, x, y, id)
end
function UIBpmEditor:draw()
	self.subs:draw()
end
function UIBpmEditor:update()
	self.subs:update()
end
UIImgButton = class(TiltedRect, function(o, x, y, buttonData, canvas)
	TiltedRect.init(o, x, y, 45*r_scale, 35*r_scale)
	o.canvas = canvas
	local c = buttonData.c or {147,146,140}
	o:setColor(c[1], c[2], c[3])
	o.sprite = sprites['ui']['white.png']
	if buttonData.align and buttonData.align == 'right' then
		o:setPosition(o.pos.x-o.w, o.pos.y)
	end
	if buttonData.id then o.canvas:registerID(buttonData.id, o) end
	-- add the overlay text
	if buttonData.img then
		o.imgSpr = buttonData.img
		local w,h = o.imgSpr:getWidth(), o.imgSpr:getHeight()
		if r_scale == 4 then
  			w,h = w*2,h*2
  		end
		o.img = Object(o.pos.x, o.pos.y, w,h)
		o.img.drawType = 'stretched'
		o.img.sprite = o.imgSpr
		o.img:setPosition(o.pos.x+o.w/2-o.img.w/2-5*r_scale, o.pos.y+o.h/2-o.img.h/2)
	end
	if buttonData.onPress then
		o.onPress = function()
			buttonData.onPress(canvas)
		end
	end
	if buttonData.url and gk then
		o.onPress = function()
			gk:openUrl(buttonData.url)
		end		
	end
	o.touchDown = CameraButton.touchDown
	o.touchUp = CameraButton.touchUp
end)
function UIImgButton:draw()
	TiltedRect.draw(self)
	if self.img then self.img:draw() end
end

