WinScreenData = class(function(o, parseObject, songData, coincount, completionCallback)
	if gk and gk:isConnected() then
		if parseObject then
			-- this is for scoring on userlevels
			o:handleUserScore(parseObject, coincount, completionCallback)
			o:handlePlayUpdate(parseObject)
		else
			-- else, we are coming from an internal level
			o:handleInternalScore(songData, coincount, completionCallback)
		end
	end
	if player.firstRun and songData.name == "A Gentle Introduction" then
		player.firstRun = false
	end
end)
-- this table keeps stuff around so it doesn't inadvertently get GC'd
local noGC = {}
function WinScreenData:countLikes(parseObject, completionCallback)
	lusers = bludParseQuery("gkUser")
	noGC[lusers] = lusers
	lusers:whereKeyEqualsObject('likes', parseObject)
	lusers:countObjects(function(count)
		completionCallback(count)
		noGC[lusers] = nil
	end) 
end
function WinScreenData:hasUserLiked(po, callback)
	-- should only vote up if the play has not already voted up this level. should probably attempt to cache that data
	-- check to see if this user has liked this level already
	lusers = bludParseQuery("gkUser")
	lusers:whereKeyEqualsObject('likes', po)
	lusers:whereKeyEquals('userId', gk:getId())
	noGC[lusers] = lusers
	lusers:countObjects(function(count, errorCode)
		if count then
			if count < 1 then
				callback(false)
			else
				callback(true)
			end
		else
			callback(nil, true)
		end
		noGC[lusers] = nil
	end)
end
function WinScreenData:voteUpLevel(po, completionCallback)
	-- should only vote up if the play has not already voted up this level. should probably attempt to cache that data
	-- check to see if this user has liked this level already
	lusers = bludParseQuery("gkUser")
	lusers:whereKeyEqualsObject('likes', po)
	lusers:whereKeyEquals('userId', gk:getId())
	lusers:countObjects(function(count, errorCode)
		if count and count < 1 then
			local relation = bludParseRelation(po, "likes")
			relation:addObject(pgkuser)
			po:incrementKey("likeCount")
			po:save(function(saved, errorCode)
				if not saved then
					completionCallback(true)
				else
					local reverseRelation = bludParseRelation(pgkuser, "likes")
					reverseRelation:addObject(po)
					pgkuser:save(function(saved, errorCode)
						completionCallback(not saved)
					end)
				end
			end)
		else
			completionCallback(true)
		end
	end)
end
function WinScreenData:handlePlayUpdate(po)
	-- should determine if we are playing our own level or not
	if po:getObject("userId") ~= gk:getId() then
		po:incrementKey("playCount")
		noGC[po] = po
		po:save(function(saved, errorCode)
			noGC[po]=nil
		end)
	end
end
function WinScreenData:handleInternalScore(songData, coincount, completionCallback, forUser)
	local pquery = bludParseQuery("Highscore")
	pquery:whereKeyEquals("userId", gk:getId())
	local index;
	if type(songData) == 'string' then
		index = songData
	else
		index = songData.worldPack..songData.name
	end
	pquery:whereKeyEquals("level_name", index)
	noGC[pquery]=pquery
	pquery:findObjects(function(objects)
		if not objects or #objects == 0 then
			local pobj = bludParseObject("Highscore")
			noGC[pobj]=pobj
			-- should determine if there are any existing levels with this name and username yet
			pobj:setObject(gk:getAlias(), "userAlias")
			pobj:setObject(gk:getId(), "userId")
			pobj:setObject("gamecenter", "portal")
			pobj:setObject(index, "level_name")
			pobj:setObject(tonumber(coincount), "score")
			pobj:save(function(saved)
				-- should probably cache the score locally and reattempt to send if this fails later
				noGC[pobj]=nil
			end)
		else
			local pobj = objects[1]
			noGC[pobj]=pobj
			if coincount >= tonumber(pobj:getObject("score")) then
				pobj:setObject(tonumber(coincount), "score")
				pobj:save(function(saved)
					-- should probably cache the score locally and reattempt to send if this fails later
					noGC[pobj]=nil
				end)
			end
		end
		-- remove the object from the table so it gets gc'd
		noGC[pquery] = nil
	end)
	WinScreenData.getHighscoreInternal(self, songData, completionCallback, forUser)
end
function WinScreenData:getHighscoreInternal(songData, completionCallback, forUser)
	-- get the highest scores from the server
	-- this should probably be relatively agressively cached
	if type(songData) == 'string' then
		index = songData
	else
		index = songData.worldPack..songData.name
	end
	local highquery = bludParseQuery("Highscore")
	highquery:whereKeyEquals("level_name", index)
	if forUser then
		highquery:whereKeyEquals("userId", gk:getId())
	end
	highquery:descendingOrder("score")
	-- highquery:addAscendingOrder("score")
	noGC[highquery]=highquery
	highquery:findObjects(function(objects)
		if objects and #objects > 0 then
			completionCallback(objects[1]:getObject("userAlias"), objects[1]:getObject("score"))
		end
		noGC[highquery]=nil
	end)
end
function WinScreenData:handleUserScore(parseObject, coincount, completionCallback, forUser)
	local pquery = bludParseQuery("Highscore")
	noGC[pquery]=pquery
	pquery:whereKeyEquals("userId", gk:getId())
	if type(parseObject)=='string' then
		pquery:whereKeyEqualsObjectWithClassAndId("level", "PlayerLevel", parseObject)
	else
		pquery:whereKeyEqualsObject("level", parseObject)
	end		
	pquery:findObjects(function(objects)
		local pobj
		if not objects or #objects == 0 then
			pobj = bludParseObject("Highscore")
			noGC[pobj]=pobj
			-- should determine if there are any existing levels with this name and username yet
			pobj:setObject(gk:getAlias(), "userAlias")
			pobj:setObject(gk:getId(), "userId")
			pobj:setObject("gamecenter", "portal")
			if type(parseObject)=='string' then
				pquery:setObjectWithClassAndId("PlayerLevel", parseObject, "level")
			else
				pobj:setParseObject(parseObject, "level")
			end		
			pobj:setObject(tonumber(coincount), "score")
			pobj:save(function(saved)
				-- should probably cache the score locally and reattempt to send if this fails later
				noGC[pobj]=nil
			end)
		else
			pobj = objects[1]
			noGC[pobj]=pobj
			if coincount >= pobj:getObject("score") then
				pobj:setObject(tonumber(coincount), "score")
				pobj:save(function(saved)
					-- should probably cache the score locally and reattempt to send if this fails later
					noGC[pobj]=nil
				end)
			end
		end
		noGC[pquery]=nil
	end)
	WinScreenData.getHighscoreUser(self, parseObject, completionCallback, forUser)
end
function WinScreenData:getHighscoreUser(parseObject, completionCallback, forUser)
	-- get the highest scores from the server
	-- this should probably be relatively agressively cached
	local highquery = bludParseQuery("Highscore")
	if forUser then
		highquery:whereKeyEquals("userId", gk:getId())
	end
	if type(parseObject)=='string' then
		highquery:whereKeyEqualsObjectWithClassAndId("level", "PlayerLevel", parseObject)
	else
		highquery:whereKeyEqualsObject("level", parseObject)
	end		
	highquery:descendingOrder("score")
	noGC[highquery]=highquery
	highquery:findObjects(function(objects)
		if objects and #objects > 0 then
			completionCallback(objects[1]:getObject("userAlias"), objects[1]:getObject("score"))
		end
		noGC[highquery]=nil
	end)
end
-- need to make this into more of an opaque class for getting and setting scores on the server
-- i.e. decouple it from the graphical representation


--[[


	if parseObject then
		local pquery = bludParseQuery("Highscore")
		pquery:whereKeyEqualsObject("level", parseObject)
		pquery:addDescendingOrder("score")
		pquery:findObjects(function(objects)
			if objects and #objects > 0 then
				o:add(TiltedButton(0,210*r_scale, objects[1]:getObject("userAlias") .. ": " .. objects[1]:getObject("score") ,  "HighScore"))
			end
		end)
	else


	-- store the players score on the server, and get the highscore list from the server
]]