BackgroundLoopState = class(Group, function(o)
	Group.init(o)
	o.items = o:add(Group())
	o.displayScale = 0.75
	o.fogAmount = 0.5
	o.w = bludG.camera.w*o.displayScale
	o.h = bludG.camera.h*o.displayScale
	o.displayObj = Object(bludG.camera.w/2-o.w/2, bludG.camera.h/2-o.h/2,o.w,o.h)
	o.stepDisplay = o:add(Object(0,0,o.w/16,o.h))
	o.stepDisplay.sprite = sprites['additive']['white.png']
	o.stepDisplay.sheet = sheets['additive']
	o.stepDisplay:setColor(255, 255, 255, 0)
	o.stepDisplay.drawType = 'stretched'
	o:setStep(1)
end)
function BackgroundLoopState:setStep(step)
	step = (step-1)%16
	self.currentStep = step
	self.stepDisplay:setPosition(self.displayObj.pos.x+self.w/16*(step), self.displayObj.pos.y)
end
function BackgroundLoopState:addItem(x, y, synth)
	-- since this takes the item in terms of quantized position, we need to convert
	local ix = self.displayObj.w/16*(x-1)
	local iy = self.displayObj.h/7*(y-1)
	local s = synth
	local i = self.items:add(SynthObject(ix+self.displayObj.pos.x, iy+self.displayObj.pos.y,s))
	i.scale = self.displayScale
	i:setPosition(i.pos.x-i.w/2, i.pos.y-i.h/2)
	i.originalPos = {x, y}
	-- set the tint of the item based on the background color
	local fogAmount = self.fogAmount
	local bgc = BackgroundData.under_color
	local t = Tweener.equations.easeNone
	for count,v in ipairs({'r', 'g', 'b'}) do
		i.tint[v] = t(fogAmount, i.tint[v], bgc[count]-i.tint[v], 1)
	end
end
function BackgroundLoopState:moveLayer(scale, fog)
	self.displayScale = scale
	self.fogAmount = fog
	self.w = bludG.camera.w*self.displayScale
	self.h = bludG.camera.h*self.displayScale
	self.displayObj = Object(bludG.camera.w/2-self.w/2, bludG.camera.h/2-self.h/2,self.w,self.h)
	self.items:call(function(v)
		if v.posTween then Tweener:removeTween(v.posTween) end
		local ix = self.displayObj.w/16*(v.originalPos[1]-1)
		local iy = self.displayObj.h/7*(v.originalPos[2]-1)
		v.posTween = Tweener:addTween(v.pos, {x=ix+self.displayObj.pos.x-v.w/2, y=iy+self.displayObj.pos.y-v.h/2})
		v.scale = self.displayScale
		local fogAmount = self.fogAmount
		local bgc = BackgroundData.under_color
		local t = Tweener.equations.easeNone
		for count,component in ipairs({'r', 'g', 'b'}) do
			v.tint[component] = t(fogAmount, v.tint[component], bgc[count]-v.tint[component], 1)
		end
	end)
	self.stepDisplay.w = self.w/16
	self.stepDisplay.h =self.h
	self:setStep(self.currentStep)
end