function getLineForUser(v)
	return {
				['t'] = 'UIline';
				['c'] = bgc(1);
				['paddingBottom']=15*r_scale;
				['subs']={
					{
						['t']='UIlabel';
						['align']='left';
						['text']=v:getObject("userAlias");
					},
					{
						['t']='UIspacer';
						['align']='left';
					},
					{
						['t']='UIlabel';
						['align']='left';
						['text']=v:getObject("score");
					}
				};
			}
end
HighscoreState = class(Group, function(o, parseObject, songData, parent)
	Group.init(o)
	o:setup(parseObject, songData, parent)
end)
function HighscoreState:setup(parseObject, songData, parent)
	local o = self
	local existingCamY = bludG.camera.scroll.y
	bludG.camera:zoomTo(0,0,1)
	self.parseObject = parseObject
	self.songData = songData
	o.bg = o:add(Object(0,0,bludG.camera.w,bludG.camera.h))
	o.bg.sprite = sprites['ui']['white.png']
	o.bg.drawType = 'stretched'
	o.bg:setColor(getColor())
	o.bg:setScrollFactor(Vec2(0,0))
	o.buttons = o:add(Group())
	o.scoreDisplay = o:add(Group())
	o.globalUiData = {}
	o.friendUiData = {}
	o.callbackCount = 2
	
	o.friendOffset = 0
	o.globeOffset = 0

	o.limit = 10
	o.queryLimit = 40

	local pos = Vec2(0,65)
	WorldSelectMenu.addTopBar(o, 'Leaderboard', function()
		bludG.camera:zoomTo(0,existingCamY,1)
		mainState:removeState(o)
	end)
	local cancelTables = {}
	if parseObject then
		table.insert(cancelTables, GetGlobalHighscoresUser(parseObject, function(objs,count, errorCode)
			o:displayGlobalScores(objs,count, errorCode)
		end,0,o.queryLimit))
		table.insert(cancelTables, GetFriendHighscoresUser(parseObject, function(objs,count, errorCode)
			o:displayFriendScores(objs,count, errorCode)
		end,0,o.queryLimit))
	else
		table.insert(cancelTables, GetGlobalHighscoresInternal(songData,function(objs,count, errorCode)
			o:displayGlobalScores(objs,count, errorCode)
		end, 0, o.queryLimit))
		table.insert(cancelTables, GetFriendHighscoresInternal(songData, function(objs,count, errorCode)
			o:displayFriendScores(objs,count, errorCode)
		end,0,o.queryLimit))
	end
	self.loadingOverlay = self:add(LoadingState(nil, function()
		for i,v in ipairs(cancelTables) do
			v.cancel()
		end
		bludG.camera:zoomTo(0,existingCamY,1)
		mainState:removeState(o)
	end))
	SongSelectMenu.setupDragArea(o)
end
function HighscoreState:displayFriendScores(objects, count, errorCode)
	local o = self
	-- if we failed, then just switch to a display showing this
	if not self.showingError then
		if not objects then
			self.showingError = true
			if self.loadingOverlay then self:remove(self.loadingOverlay); self.loadingOverlay = nil end
			local errorText = "error"
			if type(errorCode) == 'string' then
				errorText = errorText .. ": " .. errorCode
			else
				errorText = errorText .. " code: " .. errorCode
			end
			mainState:addState(ErrorState(function()					
				bludG.camera:zoomTo(0,0,1)
				mainState:removeState(o)
			end, errorText))
		else
			if self.friendObjects == nil then
				self.friendObjects = objects
			else
				-- append the scores to the end of the table
				for i,v in ipairs(objects) do
					table.insert(self.friendObjects, v)
				end
			end
			o.friendCount = count
			o:buildDisplayTable('friendUiData', 'friendOffset', 'friendCount', 'friendObjects', {GetFriendHighscoresUser, GetFriendHighscoresInternal, self.displayFriendScores}, 'friend')
			o:completedCallback()
		end
	end
end
function HighscoreState:displayGlobalScores(objects, count, errorCode)
	local o = self
	-- if we failed, then just switch to a display showing this
	if not self.showingError then
		if not objects then
			self.showingError = true
			if self.loadingOverlay then self:remove(self.loadingOverlay); self.loadingOverlay = nil end
			local errorText = "error"
			if type(errorCode) == 'string' then
				errorText = errorText .. ": " .. errorCode
			else
				errorText = errorText .. " code: " .. errorCode
			end
			mainState:addState(ErrorState(function()					
				bludG.camera:zoomTo(0,0,1)
				mainState:removeState(o)
			end, errorText))
		else
			if self.globalObjects == nil then
				self.globalObjects = objects
			else
				-- append the scores to the end of the table
				for i,v in ipairs(objects) do
					table.insert(self.globalObjects, v)
				end
			end
			o.globeCount = count
			o:buildDisplayTable('globalUiData', 'globeOffset', 'globeCount', 'globalObjects', {GetGlobalHighscoresUser, GetGlobalHighscoresInternal, self.displayGlobalScores}, 'global')
			o:completedCallback()
		end
	end
end
function HighscoreState:buildDisplayTable(uiTable, offset, count, objects, fngroup, text)
	self[uiTable] = {}
	local o = self
	-- check to see if we need to get more data
	if math.min(self[offset]+self.limit, self[count]) > #self[objects] then
		self.callbackCount = self.callbackCount+1
		-- get more data
		if parseObject then
			fngroup[1](self.parseObject, function(objs,count, errorCode) fngroup[3](self,objs,count, errorCode) end, #self[objects], self.queryLimit)
		else
			fngroup[2](self.songData, function(objs,count, errorCode) fngroup[3](self,objs,count, errorCode) end, #self[objects], self.queryLimit)
		end
		-- pop a loading screen and return
		if self.loadingOverlay then self:remove(self.loadingOverlay); self.loadingOverlay = nil end
		self.loadingOverlay = self:add(LoadingState())
		return
	end
	o.gtopline = {
		['t'] = 'UIline';
		['c'] = {getColor()};
		['paddingBottom']=15*r_scale;
		['subs']={
			{
				['t']='UIlabel';
				['align']='left';
				['text']=text .. " highscores " .. (self[offset]+1) .. "-" .. math.min(self[count], (self[offset]+self.limit)) .. " of " .. self[count];
			}
		};
	}
	if self[offset]+self.limit < self[count] then
		table.insert(o.gtopline.subs, {
			['t']='UIbutton';
			['align']='right';
			['text']="next";
			['onPress']=function()
				self.callbackCount = 1
				self[offset] = self[offset]+self.limit
				self:buildDisplayTable(uiTable, offset, count, objects, fngroup, text)
				o:completedCallback()
			end;
		})
	end
	if self[offset]+1 > 1 then
		table.insert(o.gtopline.subs, {
			['t']='UIbutton';
			['align']='right';
			['text']="prev";
			['onPress']=function()
				self.callbackCount = 1
				self[offset] = self[offset]-self.limit
				self:buildDisplayTable(uiTable, offset, count, objects, fngroup, text)
				o:completedCallback()
			end;
		})
	end
	table.insert(self[uiTable], o.gtopline)
	if self[objects] and #self[objects] > 0 then
		for i=self[offset]+1,math.min(self[count], self[offset]+self.limit) do
			local v = self[objects][i]
			local line = getLineForUser(v)
			table.insert(self[uiTable], line)
		end
	else
		table.insert(o.gtopline.subs, {
			['t']='UIspacer';
			['align']='left';
		});
		table.insert(o.gtopline.subs, {
			['t']='UIlabel';
			['align']='left';
			['text']="no " .. text .. " scores found";
		});
	end
end
function HighscoreState:completedCallback()
	self.callbackCount = self.callbackCount-1
	self.displayData = {}
	if self.callbackCount == 0 then
		if self.loadingOverlay then self:remove(self.loadingOverlay); self.loadingOverlay = nil end
		-- combine the two sets of data and display onscreen
		for i,v in ipairs(self.friendUiData) do
			table.insert(self.displayData, v)
		end
		for i,v in ipairs(self.globalUiData) do
			table.insert(self.displayData, v)
		end
		self.displayData['paddingTop']=50*r_scale;
		if self.canvas then self.scoreDisplay:remove(self.canvas) end
		self.canvas = self.scoreDisplay:add(UICanvas(self.displayData))
		SongSelectMenu.setDragMax(self, self.canvas.h)
	end
end
-- don't allow fallthrough
function HighscoreState:touchDown(x, y, id)
	Group.touchDown(self, x, y, id)
	return false
end
function HighscoreState:touchMoved(x, y, id)
	Group.touchMoved(self, x, y, id)
	return false
end
function HighscoreState:touchUp(x, y, id)
	Group.touchUp(self, x, y, id)
	return false
end
local noGC = {}
function GetFriendHighscoresInternal(songData, completionCallback, offset, limit)
	-- this function and the ones below should return a table
	-- with a single function that cancels the ongoing query
	-- the reason that it is a table is so that we can maintain
	-- a reference to the function to overwrite it
	local canceled = false
	local returnData = {['cancel']=function()
		canceled = true
	end}
	gk:getFriends(function(friends)
		if canceled then
			return
		end
		local friends = assert(loadstring(friends))()
		-- get the highest scores from the server
		-- this should probably be relatively agressively cached
		local highquery = bludParseQuery("Highscore")
		highquery:whereKeyEquals("level_name", songData.worldPack..songData.name)
		highquery:addDescendingOrder("score")
		highquery:addDescendingOrder("updatedAt")
		if offset then
			highquery:skip(offset)
		end
		if limit then
			highquery:limit(limit)
		end
		local playerIds = {}
		for i,v in ipairs(friends) do
			table.insert(playerIds, v.playerID)
		end
		highquery:whereKeyContainedIn("userId", playerIds)
		noGC[highquery]=highquery
		highquery:findObjects(function(objects, errorCode)
			if not objects then
				completionCallback(objects, nil, errorCode)
			else
				highquery:countObjects(function(count)
					completionCallback(objects, count)
					noGC[highquery]=nil
				end)
			end
		end)
		returnData['cancel'] = function()
			if noGC[highquery] then 
				noGC[highquery]:cancel()
			end
			noGC[highquery]=nil
		end
	end)
	return returnData
end
function GetFriendHighscoresUser(parseObject, completionCallback, offset, limit)
	local canceled = false
	local returnData = {['cancel']=function()
		canceled = true
	end}
	gk:getFriends(function(friends)
		local friends = assert(loadstring(friends))()
		-- get the highest scores from the server
		-- this should probably be relatively agressively cached
		local highquery = bludParseQuery("Highscore")
		highquery:whereKeyEqualsObject("level", parseObject)
		highquery:addDescendingOrder("score")
		highquery:addDescendingOrder("updatedAt")
		local playerIds = {}
		for i,v in ipairs(friends) do
			table.insert(playerIds, v.playerID)
		end
		highquery:whereKeyContainedIn("userId", playerIds)
		if offset then
			highquery:skip(offset)
		end
		if limit then
			highquery:limit(limit)
		end
		noGC[highquery]=highquery
		highquery:findObjects(function(objects, errorCode)
			if not objects then
				completionCallback(objects, nil, errorCode)
			else
				highquery:countObjects(function(count)
					completionCallback(objects, count)
					noGC[highquery]=nil
				end)
			end
		end)
		returnData['cancel'] = function()
			if noGC[highquery] then 
				noGC[highquery]:cancel()
			end
			noGC[highquery]=nil
		end
	end)
	return returnData
end
function GetGlobalHighscoresInternal(songData, completionCallback, offset, limit)
	-- get the highest scores from the server
	-- this should probably be relatively agressively cached

	local highquery = bludParseQuery("Highscore")
	highquery:whereKeyEquals("level_name", songData.worldPack..songData.name)
	highquery:addDescendingOrder("score")
	highquery:addDescendingOrder("updatedAt")
	if offset then
		highquery:skip(offset)
	end
	if limit then
		highquery:limit(limit)
	end
	noGC[highquery]=highquery
	highquery:findObjects(function(objects, errorCode)
		if not objects then
			completionCallback(objects, nil, errorCode)
		else
			highquery:countObjects(function(count)
				completionCallback(objects, count)
				noGC[highquery]=nil
			end)
		end
	end)
	return {['cancel']=function()
		if noGC[highquery] then
			noGC[highquery]:cancel()
			noGC[highquery] = nil
		end
	end}
end
function GetGlobalHighscoresUser(parseObject, completionCallback, offset, limit)
	-- get the highest scores from the server
	-- this should probably be relatively agressively cached
	local highquery = bludParseQuery("Highscore")
	highquery:whereKeyEqualsObject("level", parseObject)
	highquery:addDescendingOrder("score")
	highquery:addDescendingOrder("updatedAt")
	if offset then
		highquery:skip(offset)
	end
	if limit then
		highquery:limit(limit)
	end
	noGC[highquery]=highquery
	highquery:findObjects(function(objects, errorCode)
		if not objects then
			completionCallback(objects, nil, errorCode)
		else
			highquery:countObjects(function(count)
				completionCallback(objects, count)
				noGC[highquery]=nil
			end)
		end
	end)
	return {['cancel']=function()
		if noGC[highquery] then
			noGC[highquery]:cancel()
			noGC[highquery] = nil
		end
	end}
end