--[[
	This is the state that shows up above most of the screens that are
	loading data from the web when they need to block the interface.
	
	The cancelFn should stop the loading action and remove the state that triggered the load.
--]]

LoadingState = class(Group, function(o, text, cancelFn)
	Group.init(o)
	local displaytext = text or "loading"
	local bg = o:add(Object(0,0,bludG.camera.w, bludG.camera.h))
	bg:setColor(0,0,0,200)
	bg.sprite = sprites['ui']['white.png']
	bg.drawType = 'stretched'
	bg.scrollFactor = Vec2(0,0)
	o:add(LoadingHero(bludG.camera.w/2, bludG.camera.h/2))
	local loadingText = o:add(StringObject(0,0,displaytext))
	loadingText:setPosition(bludG.camera.w/2-loadingText.w/2, bludG.camera.h/2-loadingText.h/2)
	loadingText:setColor(255, 255, 255)
	loadingText:setScrollFactor(Vec2(0,0))
	if cancelFn then
		local cancelLine = {{
			['t'] = 'UIline';
			['paddingBottom']=15*r_scale;
			['subs']={
				{
					['t']='UIbutton';
					['text']="Cancel";
					['align']='left';
					['onPress']=function()
						cancelFn()
					end;
				}
			}
		}};
		o.canvas = o:add(UICanvas(cancelLine))
	end
end)
function LoadingState:touchDown(x, y, id)
	Group.touchDown(self, x, y, id)
	return false
end
function LoadingState:touchMoved(x, y, id)
	Group.touchMoved(self, x, y, id)
	return false
end
function LoadingState:touchUp(x, y, id)
	Group.touchUp(self, x, y, id)
	return false
end
LoadingHero = class(CenteredObject, function(o, x, y)
	CenteredObject.init(o, x, y, 10, 10)
	o.sprite = sprites['ui']['head.png']
	o.last = o.pos:copy()
	o.scrollFactor = Vec2(0,0)
	o.center = o.pos:copy()
	o.trail = TrailObject(sprites['ui']['white.png'], sheets['ui'], 20)
	o.trail:setColor(getYellow())
	o.timer = 0
	for i=1,20 do
		o:update()
	end
end)
function LoadingHero:update()
	self.timer = self.timer + 1/50
	CenteredObject.update(self)
	self.last:set(self.pos)
	self.pos.x = self.center.x + math.cos(self.timer*2)*150*r_scale
	self.pos.y = self.center.y + math.sin(self.timer*4)*50*r_scale
	-- self.rot = -bludG.last_t*120
	self.rot = math.deg(-self.pos:copy():sub(self.last):normalize():angle()+(math.pi/2))
	self.trail:addPoint(self.pos.x, self.pos.y, 10*r_scale)
	self.trail:modWidth(0.9)
end
function LoadingHero:draw()
	self.trail:draw(0,0,1)
	CenteredObject.draw(self)
end