local bpm = 100
function HSVtoRGB(h, s, v)

	local r, g, b

	local i = math.floor(h * 6)
	local f = h * 6 - i
	local p = v * (1 - s)
	local q = v * (1 - f * s)
	local t = v * (1 - (1 - f) * s)

	local switch = i % 6
	if switch == 0 then
	r = v g = t b = p
	elseif switch == 1 then
	r = q g = v b = p
	elseif switch == 2 then
	r = p g = v b = t
	elseif switch == 3 then
	r = p g = q b = v
	elseif switch == 4 then
	r = t g = p b = v
	elseif switch == 5 then
	r = v g = p b = q
	end

	return math.floor(r*255), math.floor(g*255), math.floor(b*255)
	
end
SH3EditState = class(Group, function(o, songData, slot_id)
	Group.init(o)
	o.maxBars = 32
	bludG.camera:zoomTo(0,0,1)
	local id = slot_id
	-- if the song data is a string, that means that we need to generate a new song
	if type(songData) == 'string' then
		o.songData = {{}}
		-- come up with a random name for the song
		-- should eventually prefix this with the players
		o.songData.filename = "/player_levels/level_"..id..".lua"
		o.songData.name = songData
		o.songData.slot_id = id
		o.songData.worldPack = player.currentWorld
		-- pull the default bpm from the synth data for this world
		o.songData.bpm = SynthData.bpm
	else
		o.songData = songData
		o.songData.slot_id = id
		if not o.songData.bpm then o.songData.bpm = SynthData.bpm end
	end
	o:setup()
	player:startTimer()
	o:rebuildPatternDisplay()
	o:saveSong()
end)
function SH3EditState:setup()
	-- add in the background
	if BackgroundData then
		self.vectorBackground = self:add(VectorBackground())
	end
	local o = self
	o.selectorHeight = bludG.camera.h/9
	player:setBpm(o.songData.bpm)
	o.selectorWidth = bludG.camera.w/16
	-- add an underlying grid
	o.grid = o:add(Group())
	o.gridHash = {}
	for gx=0,15 do
		for gy=0,6 do
			local g = o.grid:add(Object(gx*bludG.camera.w/16+2*r_scale, gy*o.selectorHeight+2*r_scale+o.selectorHeight, o.selectorWidth-4*r_scale, o.selectorHeight-4*r_scale))
			g.sprite = sprites['ui']['white.png']
			g.drawType = 'stretched'
			o.gridHash[gx .. "_" .. gy] = g
			local a = 20
			if gx%4 == 0 then
				a = 60
			end
			g.targetA = a
			g:setColor(255, 255, 255,a)
		end
	end
	o.stepDisplay = o:add(Group())
	o.beatCount = 0
	o.loopData = o.songData[1]
	o.timerBar = o:add(Object(0,0,bludG.camera.w/16, bludG.camera.h))
	o.timerBar.sprite = sprites['ui']['white.png']
	o.timerBar.sheet = sheets['ui']
	o.timerBar.drawType = 'stretched'
	o.timerBar:setColor(255,255,255,100)
	local toggleButton = o:add(DragArea(0,o.selectorHeight,bludG.camera.w, bludG.camera.h-o.selectorHeight*2))
	toggleButton.sprite = sprites['ui']['white.png']
	toggleButton.drawType = 'stretched'
	toggleButton.onPress = function(da, x, y, id)
		o:editPress(da, x, y, id)
	end
	toggleButton.onStart = function(da, x, y, id)
		o:editMove(da, x, y, id, false)
	end
	toggleButton.onMove = function(da, x, y, id)
		o:editMove(da, x, y, id)
	end
	toggleButton:setColor(120, 180, 160, 0)
	toggleButton.scrollFactor = Vec2(0,0)
	o.toggleButton = toggleButton

	-- add the sound selectors
	o:addPlayButton()
	o:addBarDisplay()
	o:addPatternButtons()
	o.soundSelectors = o:add(Group())
	self:addSelectors()
	o:add(DrumMachineOverlay(o))

	o.bg_scrollx = 0
end
function SH3EditState:addSelectors()
  self.bonusOn = false
  local o = self
  for i=1,#SynthData do
    local sd = SynthData[i]
    local selector = o.soundSelectors:add(SynthObject((i)*bludG.camera.w/(#SynthData+1), 0, sd))
    ObjectToButton(selector)
    selector.noParticle = true
    selector.count = SynthData[i].id or i
    selector.onPress = function()
      self.soundSelectors:call(function(v)
      	if v:is_a(SynthObject) then
        	v.pos.y = 0
        end
      end)
      o.currentSelection = selector
      o.currentSelection.pos.y = 10*r_scale
    end
  end
  -- add in a button to mark synths as key synths
  -- the bonus should mark all the existing ones as bonus or non bonus depending
  local bonus = o.soundSelectors:add(NonBonusObj((0)*bludG.camera.w/(#SynthData+1)+8*r_scale, 8*r_scale, o.selectorHeight*0.75, o.selectorHeight*0.75))
  ObjectToButton(bonus)
  bonus.noParticle = true
  bonus.onPress = function()
    if self.bonusOn then
      self.bonusOn = nil
      bonus.sprite = sprites['ui']['switch_2.png']
    else
      self.bonusOn = true
      bonus.sprite = sprites['ui']['switch_1.png']
    end
    self.soundSelectors:call(function(v)
      if v:is_a(SynthObject) and not v.dataFile.killer then
        v:setKey(self.bonusOn)
      end
    end)
  end
  bonus.onPress()
  o.soundSelectors.members[1].onPress()
end
function SH3EditState:editMove(da, x, y, id, noPreview)
	local o = self
	-- quantize and add the object
	local stepPos = math.floor(x/bludG.camera.w*16)
	local vPos = math.floor((y-da.pos.y)/da.h*7)
	if vPos < 0 or vPos > 6 then
		return
	end
	if self.currentSelection == 'bonus' then
		return
	end
	if not self.loopData[stepPos+1] or not self.loopData[stepPos+1][vPos] then
		if not noPreview and not self.previewTrigger or not (self.previewTrigger[1]==stepPos and self.previewTrigger[2]==vPos) then
			-- flash the grid cell that you are tapping on
			local g = self.gridHash[stepPos .. "_" .. vPos]
			if g.colorTween then Tweener:removeTween(g.colorTween) end
			g.tint.a = 200
			g.colorTween = Tweener:addTween(g.tint, {a=g.targetA})
			-- play the sound for the grid cell
			self.previewTrigger = {}
			self.previewTrigger[1]=stepPos
			self.previewTrigger[2]=vPos
			pd:addTrigger(player:getSampleAndNote(SynthData.getId(o.currentSelection.count), 7-vPos))
		end
	end
end
function SH3EditState:editPress(da, x, y, id)
	self.previewTrigger = nil
  local o = self
  -- quantize and add the object
  local stepPos = math.floor(x/bludG.camera.w*16)
  x = stepPos*bludG.camera.w/16
  local vPos = math.floor((y-da.pos.y)/da.h*7)
  if vPos < 0 or vPos > 6 then
    return
  end
  y = vPos*da.h/7+da.pos.y
  -- check to see if we should add or remove
  if o.loopData[stepPos+1] and o.loopData[stepPos+1][vPos] or self.currentSelection == 'bonus' then
    -- removing
    -- if we are on the bonus thing
    self.stepDisplay:remove(o.loopData[stepPos+1][vPos])
    o.loopData[stepPos+1][vPos] = nil
  else
    --adding
    local step = o.stepDisplay:add(SynthObject(x, y, SynthData.getId(o.currentSelection.count)))
    step.drawType = 'stretched'
    step.currentSelection = o.currentSelection
    if not o.loopData[stepPos+1] then
      o.loopData[stepPos+1] = {}
    end
    o.loopData[stepPos+1][vPos] = step
    if not o.loopData[stepPos+1][vPos].dataFile.killer then
      o.loopData[stepPos+1][vPos]:setKey(self.bonusOn)
    end
  end
  self:saveSong()
end
function SH3EditState:saveSong()
  local cleanSongData = self:cleanSongData(self.songData)
  persistence.store(blud.doc_root .. self.songData.filename, cleanSongData);
end
function SH3EditState:cleanSongData( ... )
  local cleanSongData = {}
  
  for li,loop in ipairs(self.songData) do
    cleanSongData[li] = {}
    for si,step in pairs(loop) do
      if type(si) == 'number' then
      cleanSongData[li][si] = {}
        for k, v in pairs(step) do
          if v.currentSelection then
            local isKey = 0
            if v.key then isKey = 1 end
            cleanSongData[li][si][k] = {v.currentSelection.count, k, isKey}
          else
            cleanSongData[li][si][k] = {v[1], v[2], v[3]}
          end
          print(li, cleanSongData[li][si][k][3])
        end
      end
    end
  end
  -- remove all the empty patterns from the end
  for li=#cleanSongData,1,-1 do
    local hasObj = false
    for si,step in pairs(cleanSongData[li]) do
        for k, v in pairs(step) do
          hasObj = true
        end
    end
    if not hasObj then
      cleanSongData[li] = nil
    else
      break;
    end
  end
  cleanSongData.slot_id = self.songData.slot_id
  cleanSongData.filename = self.songData.filename
  cleanSongData.worldPack = self.songData.worldPack
  cleanSongData.name = self.songData.name
  cleanSongData.bpm = self.songData.bpm

  return cleanSongData
end
function SH3EditState:addPlayButton()
  self.playbuttonPress = function()

    player.currentSong = self.songData
    local cleanData = self:cleanSongData()
    if cleanData[1] == nil then
        local text = mainState.particles:addRecycle(TextPop(bludG.camera.w/2, bludG.camera.h/2, "this level is TOOO Boring!"))
        text.text:setColor(0,0,0)
        text = mainState.particles:addRecycle(TextPop(bludG.camera.w/2, bludG.camera.h/2+20*r_scale, "add some coins"))
        text.text:setColor(0,0,0)

      return;
    end
    local sh3 = SpaceHero3(self:cleanSongData(), nil, self)
    mainState = WrapperState(Group());
    bludG.camera:zoomTo(0,0)
    mainState:changeState(sh3)
  end
end
function SH3EditState:addPatternButtons()
	self.currentPattern = 1
	self.nextPatternPress = function()
		if self.bgScroll then Tweener:removeTween(self.bgScroll) end
		self.currentPattern = math.min(self.currentPattern+1, self.maxBars)
		self:rebuildPatternDisplay()
		self.bgScroll = Tweener:addTween(self, {bg_scrollx=(self.currentPattern-1)*bludG.camera.w}, {func='easeNone', time=0.25})
	end
	self.prevPatternPress = function()
		if self.bgScroll then Tweener:removeTween(self.bgScroll) end
		self.currentPattern = math.max(self.currentPattern-1, 1)
		self:rebuildPatternDisplay()
		self.bgScroll = Tweener:addTween(self, {bg_scrollx=(self.currentPattern-1)*bludG.camera.w}, {func='easeNone', time=0.25})
	end
end
-- handles rebuilding the display after load or when changing patterns
function SH3EditState:rebuildPatternDisplay()
	for i=math.max(1,self.currentPattern-3),self.currentPattern do
		-- self.currentPattern = i
		if not self.songData[self.currentPattern] then self.songData[self.currentPattern] = {} end
		self.loopData = self.songData[self.currentPattern]
		-- clear all the steps off the screen
		self.stepDisplay = self:replace(self.stepDisplay, Group())
		for step,stepData in pairs(self.loopData) do
			for k,v in pairs(stepData) do
				local currentSelection = v.currentSelection
				if not currentSelection then
					-- need to loop through the sound selectors and find the one with this could
					self.soundSelectors:call(function(ss)
						if ss.count == v[1] then
							currentSelection = ss
						end
					end)
				end				
				if not currentSelection then
					break;
				else
				end

				local step = self.stepDisplay:add(SynthObject((step-1)/16*bludG.camera.w, k/7*self.toggleButton.h+self.toggleButton.pos.y, SynthData.getId(currentSelection.count)))
				step.currentSelection = currentSelection
				step.drawType = 'stretched'
				if (v.currentSelection and v.key) or v[3]==1 then
					step:setKey(true)
				else
					step:setKey(false)
				end
				stepData[k] = step
			end
		end
		self.barDisplay:setValue('bar: ' .. self.currentPattern)
	end
end
function SH3EditState:addBarDisplay()
	local p = 4*r_scale
	self.barDisplay = self:add(StringObject(bludG.camera.w/4+p, self.toggleButton.pos.y+self.toggleButton.h+p, 'bar: 1', nil, 0.4))
end
function SH3EditState:paste()
	self.songData = self:cleanSongData(self.songData)
	if self.clipboard then
		local loop = self.clipboard
		local copy = {}
		for si,step in pairs(loop) do
			if type(si) == 'number' then
				copy[si] = {}
				for k, v in pairs(step) do
					if v.currentSelection then
						local isKey = 0
						if v.key then isKey = 1 end
						copy[si][k] = {v.currentSelection.count, k, isKey}
					else
						copy[si][k] = {v[1], v[2], v[3]}
					end
				end
			end
		end
		table.insert(self.songData, self.currentPattern, copy)
	end
	self:saveSong()
	self:rebuildPatternDisplay()
end
function SH3EditState:cutCurrent()
	self.songData = self:cleanSongData(self.songData)
	self.clipboard = self.songData[self.currentPattern]
	table.remove(self.songData, self.currentPattern)
	self:saveSong()
	self:rebuildPatternDisplay()
end
function SH3EditState:copyCurrent()
	self.songData = self:cleanSongData(self.songData)
	self.clipboard = self.songData[self.currentPattern]
	self:saveSong()
	self:rebuildPatternDisplay()
end
function SH3EditState:sync()
	if not self.pulsers then self.pulsers = self:add(Pulsers()) end
	-- setup some containers to hold everything
	-- check to see if we need to add anything
	local levelStep = self.beatCount%16+1
	self.timerBar:setPosition((levelStep-1)*bludG.camera.w/16, 0)
	for i=self.currentPattern-3,self.currentPattern do
	if self.songData[i] and self.songData[i][levelStep] then
		for k,v in pairs(self.songData[i][levelStep]) do
			if v.currentSelection then
				local s = SynthData.getId(v.currentSelection.count)
				if s.sound and (i==self.currentPattern or not s.killer) then
					local p = self.pulsers:addPulse(v)
					p:setColor(v.tint.r, v.tint.g, v.tint.b, 255)
					pd:addTrigger(player:getSampleAndNote(s, 7-k))
				end
			end
		end
	end   
	end
	self.beatCount = self.beatCount+1
end