SongSelectMenu = class(Group, function(o, playmode)
	bg:setBackground(getColor())
	mainbg = {}
	Group.init(o)
	o.playmode = playmode
	o:setup()
	bludG.camera:scrollTo(0,0)
end)
function SongSelectMenu:setupDragArea(maxY)
	local o = self
	if self.drag then self:remove(self.drag) end
	o.dragmaxY = maxY or bludG.camera.h+1
	o.dragmaxY = (o.dragmaxY+40*r_scale)-bludG.camera.h
	o.drag = o:add(DragArea(0,0,bludG.camera.w, bludG.camera.h))
	o.drag.onStart = function(da,x,y,id)
		if not da.dragVelocity then da.dragVelocity = Vec2(0,0) end
		bludG.camera:cancelTween()
		-- if I wanted to be able to accelerate inertial scrolling, I wouldn't clear here... but who cares!
		da.dragVelocity.x, da.dragVelocity.y = 0,0
		o.moveAmt = Vec2(0,0)
	end
	o.drag.onMove = function(da,x,y,id)
		-- should scale these things to elapsed time
		local newVel = Vec2(x-da.lastPos[id].x, y-da.lastPos[id].y)
		local dv = da.dragVelocity
		dv.x, dv.y = (newVel.x-dv.x)*0.5+dv.x, (newVel.y-dv.y)*0.5+dv.y
		-- should keep an average velocity
		SongSelectMenu.panModeHandler(self, da, x, y, id)
	end
	o.drag.update = function(da)
		if da.dragVelocity and da.fingerCount == 0 then
			local dv = da.dragVelocity
			dv.x = 0
			dv:mult(0.95)
			bludG.camera.scroll:sub(dv)
			if bludG.camera.scroll.y < 0 then
				dv:mult(0.8)
				bludG.camera:zoomTo(0,0,1)
			elseif bludG.camera.scroll.y > o.dragmaxY then
				dv:mult(0.8)
				bludG.camera:zoomTo(0,o.dragmaxY,1)
			end
		end
	end
	o.drag.onPress = function(da,x,y,id) 
		if o.moveAmt:length()<5*r_scale then
			return true
		end
		return false
	end
	o.drag.scrollFactor = Vec2(0,0)
end
function SongSelectMenu:setDragMax(max)
	self.dragmaxY = math.max(0, (max+40*r_scale)-bludG.camera.h)
end
function SongSelectMenu:present()
	bg:setBackground(getColor())
end
function SongSelectMenu:setup()
	self.offset = 0
	self.limit = 100
	self.perPage = 15
	self.songData = {}
	self.songButtons = self:add(Group())
	local backfunction = function()
		mainState = WrapperState(Group());
		mainState:changeState(GlobeModeMenu())
	end
	backfunction = self.backfunction or backfunction
	WorldSelectMenu.addTopBar(self, 'choose song', backfunction)
	self:getData()
end
local noGC = {}
function SongSelectMenu:getData()
	-- load songs from server
	pquery = bludParseQuery("PlayerLevel")
	pquery:addDescendingOrder("createdAt")
	pquery:limit(self.limit)
	pquery:skip(self.offset)
	noGC[pquery]=pquery
	-- should throw up a loading from server screen here
	pquery:findObjects(function(objects, errorCode)
		if not objects then
			self:dealWithError(errorCode)
		else
			if not self.count then
				pquery:countObjects(function(count, errorCode)
					if not count then
						self:dealWithError(errorCode)
					else
						noGC[pquery]=nil
						self.count = count
						self:dealWithReturnData(objects, self.count)
					end
				end)
			else
				noGC[pquery]=nil
				self:dealWithReturnData(objects, self.count)
			end
		end
	end)
	self.loadingOverlay = self:add(LoadingState(nil, function()
		pquery:cancel();
		bludG.camera:zoomTo(0,0,1)
		self:remove(self.loadingOverlay)
	end))
end
function SongSelectMenu:dealWithError(errorCode)
	self:remove(self.loadingOverlay)
	local errorText = "error"
	if type(errorCode) == 'string' then
		errorText = errorText .. ": " .. errorCode
	else
		errorText = errorText .. " code: " .. errorCode
	end
	mainState:addState(ErrorState(function()
		-- press the back button
		mainState = WrapperState(Group());
		mainState:changeState(GlobeModeMenu())
	end, errorText))
end
function SongSelectMenu:dealWithReturnData(data, count)
	self:remove(self.loadingOverlay)
	-- copy the query data onto the song data
	for i,v in ipairs(data) do
		table.insert(self.songData, v)
	end
	self:addSongButtons(self.songData)
end
function SongSelectMenu:addSongButtons(objects)
	-- check to see if we need more data
	if math.min(self.offset+self.perPage, self.count) > #self.songData then
		-- get more data
		self:getData()
		return
	end
	if self.canvas then self.songButtons:remove(self.canvas) end
	local uiData = {['paddingTop']=50*r_scale}
	-- add the pagination controls
	self.gtopline = {
		['t'] = 'UIline';
		['c'] = bgc(1);
		['paddingBottom']=15*r_scale;
		['subs']={
			{
				['t']='UIlabel';
				['align']='left';
				['text']="page " ..math.ceil((self.offset+1)/self.perPage) .. " of " .. math.ceil(self.count/self.perPage);
			}
		};
	}
	if self.offset+self.perPage < self.count then
		table.insert(self.gtopline.subs, {
			['t']='UIbutton';
			['align']='right';
			['text']="next";
			['onPress']=function()
				self.offset = self.offset+self.perPage
				self:addSongButtons(objects)
			end;
		})
	end
	if self.offset+1 > 1 then
		table.insert(self.gtopline.subs, {
			['t']='UIbutton';
			['align']='right';
			['text']="prev";
			['onPress']=function()
				self.callbackCount = 1
				self.offset = self.offset-self.perPage
				self:addSongButtons(objects)
			end;
		})
	end
	table.insert(uiData, self.gtopline)

	for i=self.offset+1,math.min(self.count, self.offset+self.perPage) do
		local v = self.songData[i]
		local ld = assert(loadstring(v:getObject("levelData")))()
		local line = {
			['t'] = 'UIline';
			['paddingBottom']=15*r_scale;
			['subs']={}
		}
		table.insert(line.subs, {
			['t']='UIlabel';
			['align']='left';
			['text']=v:getObject("userAlias");
		})
		table.insert(line.subs, {
				['t']='UIspacer';
				['c']={getColor()};
				['align']='left';
			}
		)
		table.insert(line.subs, {
			['t']='UIlabel';
			['align']='left';
			['text']=ld.name;
		})
		table.insert(line.subs, {
			['t']='UIbutton';
			['text']='play';
			['align']='right';
			['onPress']=function()
				-- check to see if the song exists on the player model, if not create it
				if not player.songs[v] then
					player.songs[v] = {{}}
				end
				player:loadWorldPack(ld.worldPack, nil, function()
					player.currentSong = ld
					mainState = WrapperState(Group());
					mainState:changeState(SpaceHero3(ld, v, self))
				end)
			end
		})
		table.insert(uiData, line)
	end
	self.canvas = self.songButtons:add(UICanvas(uiData))
	self:setupDragArea(self.canvas.h)
end
function SongSelectMenu:fixCamera()
	bludG.camera.scroll.y = 0
end
function SongSelectMenu:panModeHandler(da, x, y, id)
	local p = Vec2(x, y)
	p:sub(da.lastPos[id])
	p.x = 0
	self.moveAmt:add(p)
	bludG.camera.scroll:sub(p)
end
-- print('here')
-- mainState = WrapperState(Group());
-- mainState:changeState(SongSelectMenu(true))
