--http://www.network-science.de/ascii/
-- roman
SpaceHero3 = class(Group, function(o, songData, parseObject, parent)
	Group.init(o)
	o.songData = songData
	o.parent = parent
	o:setup()
	o.deadEnemies = o:add(Group())
	o.beats = o:add(Group())
	o.walls = o:add(Group())
	o.levelData = {}
	o.parseObject = parseObject
	o.levelPointCount = 0
	for i,loop in ipairs(songData) do
		o.levelData[i] = {}
		for k,v in pairs(loop) do
			o.levelData[i][k] = {}
			for vPos,item in pairs(v) do
				-- if this needs to be cleaned up, then clean it.
				if item.currentSelection then
					o.levelData[i][k][vPos] = {item.currentSelection.count, vPos}
					if item.key then
						o.levelData[i][k][vPos][3] = 1
					end
				else
					o.levelData[i][k][vPos] = item
				end
			end
		end
	end
	-- o:randomSong()
	-- insert a blank loop at the beginning
	-- this is so we can give control to the player when they start clicking
	table.insert(o.levelData, 1, {})

	-- if the song data doesn't have an internal bpm, then set it from the synth data
	if not o.songData.bpm then o.songData.bpm = SynthData.bpm end
	o:restart()
	player:setBpm(o.songData.bpm)
end)
function SpaceHero3:randomSong()
	local o = self
	o.levelData = {}
	for i,loop in ipairs(self.songData) do
		o.levelData[i] = {}
		for k,v in pairs(loop) do
			o.levelData[i][k] = {}
		end
		-- fill in the loop with random data
		local numAdding = math.random(10, 20)
		for addCount=1, numAdding do
			local step = math.random(1, 16)
			local vPos = math.random(0, 6)
			if not o.levelData[i][step] then o.levelData[i][step] = {} end
			o.levelData[i][step][vPos] = {math.random(1, 10), vPos}
			if math.random(1, 2) > 1  then
				o.levelData[i][step][vPos][3] = 1
			end				
		end
	end	
end
function SpaceHero3:restart()
	self.walls:clear()
	self.beats:clear()
	if self.winText then self:remove(self.winText) end
	self.gameOver = false
	self.currentLoop = 1
	self.deathWait = 0
	self.invincibilityCount = 0
	self.loopCoinCount = 0
	self.totalCoinCount = 0
	self.lives = self:countLoops()
	self.jetButton:clearTouches()
	self:clearHits()
	self.forceLoopOnce = 16
	self.healthBar.max = game_settings['max_health']
	self.healthBar:setValue(game_settings['starting_health'])

	player:startTimer()
	
	self.scoreMultiplier = 1
    self.score = 0
    self.scoreboard:setScore(self.score)
    self.scoreboard:setMultiplier(self.scoreMultiplier)
    self.scoreboard:jumpToTarget()
    self.hero:setPosition(65*r_scale,bludG.camera.h*0.5)
    self.hero.fingerDown = false
    self.jetOn = false
    self.shotCharge:setValue(game_settings['start_bullets'])
    self.hero:setShotCharge(game_settings['start_bullets'])
    -- add the initial wall
    if self:currentLoopHasKeys() then
    	self.currentWall = self.walls:addRecycle(Wall())
    end
end
function SpaceHero3:clearHits()
	-- clear out the coin count so far
	self.loopCoinCount = 0
	for lk, loop in pairs(self.levelData) do
		for k, step in pairs(loop) do
			for k2, item in pairs(step) do
				self.levelData[lk][k][k2][4] = nil
				item.hit = false
			end
		end
	end
end
-- returns the number of loops that have something in them
function SpaceHero3:countLoops()
	local count = 0
	for k,v in pairs(self.levelData) do
		local c = 0
		for j, n in pairs(v) do
			if n then
				c=c+1
				break;
			end
		end
		if c > 0 then
			count = count + 1
		end
	end
	return count
end
function SpaceHero3:countCoins()
	local count = 0
	for lk, loop in pairs(self.levelData) do
		for k, step in pairs(loop) do
			for k2, item in pairs(step) do
				if item then
					if not SynthData.getId(item[1]).killer then
						count = count + 1
					end
				end
			end
		end
	end
	return count
end
--[[

 .oooooo..o oooooooooooo ooooooooooooo ooooo     ooo ooooooooo.   
d8P'    `Y8 `888'     `8 8'   888   `8 `888'     `8' `888   `Y88. 
Y88bo.       888              888       888       8   888   .d88' 
 `"Y8888o.   888oooo8         888       888       8   888ooo88P'  
     `"Y88b  888    "         888       888       8   888         
oo     .d8P  888       o      888       `88.    .8'   888         
8""88888P'  o888ooooood8     o888o        `YbodP'    o888o        

]]
function SpaceHero3:setup()
	local o = self
	-- ipad conversion
	local screenRatio = 320/480
	local height = bludG.camera.h
	local heightDiff = 0
	if ipad == "_ip5" then
		screenRatio = 640/1136
	end
	if ipad ~= "" then
		height = bludG.camera.w*screenRatio
		heightDiff = bludG.camera.h-height
	end
	o.gridHeight = height/8
	o.topOffset = o.gridHeight/2+heightDiff/2
	-- convert the top offset on ipad to add letterboxing

	self:setupUnderBg()

	bludG.camera:zoomTo(0,0,1)
	o.ground = o:add(Object(0,height-o.topOffset+heightDiff/2, bludG.camera.w, 10))
	o.ground.sprite = sprites['ui']['white.png']
	o.ground.drawType = 'stretched'
	o.ground:setColor(100, 255, 100,0)

	o.ceiling = o:add(Object(0,0, bludG.camera.w, o.topOffset))
	o.ceiling.sprite = sprites['ui']['white.png']
	o.ceiling.drawType = 'stretched'
	o.ceiling:setColor(100, 255, 100,0)

	self.hero = HeroObject(65*r_scale,bludG.camera.h*0.5)
	self.gravity = Vec2(0,360*r_scale)
	self.bullets = self:add(Group())
	o.beatCount = 0
	self:addControls()
	self.bg_scrollx = 0
	self.nonBeat = self:add(Group())
	self.deathWait = 0
	self.healthBar = HealthBar(0,0,bludG.camera.w,20*r_scale)
	self.playOverlay = o:add(PlayOverlay(self))
	self.scoreboard = o.playOverlay.scoreboard
	self:addPauseButton()

	-- only show the tutorial on the first level
	if self.songData.loadTutorial then
		self:add(TutorialOverlay(self))
	else
		self:add(IntroControl(self))
	end
	
	self:add(self.hero)
	self.backgroundLoop = self:add(BackgroundSynthStack())
	local cmW = 80*r_scale
	local cmH = 10*r_scale
	self.shotCharge = ChargeMeter(bludG.camera.w/2-cmW/2, bludG.camera.h-cmH, cmW, cmH)
	self.shotCharge.max = game_settings['max_bullets']
	self.shotCharge.value = 2
	self.hero:setShotCharge(self.shotCharge.value)
end
function SpaceHero3:addControls()
	self.jetButton = self:add(self:newToggle(bludG.camera.w/2,0,bludG.camera.w/2, bludG.camera.h))
	self.hero.jetacceleration = Vec2(0,-700*r_scale)
	self.jetOn = false
	self.jetButton.onStart = function()
		if self.jetButton.fingerCount == 1 then
			self.hero.fingerDown = true
		end
		return true
	end
	self.jetButton.onPress = function()
		if self.jetButton.fingerCount == 0 then
			self.hero.fingerDown = false
		end
		return true
	end
	self.shotButton = self:add(self:newToggle(0,0,bludG.camera.w/2, bludG.camera.h))
	self.shotButton.onStart = function()
		self:fireBullet()
		return true
	end
end
function SpaceHero3:addPauseButton()
	self.playOverlay.pause.onPress = function()
		self.paused = true
		pd:startList("fromOF")
		pd:addSymbol("stopTimer")
		pd:addFloat(player.currentBpm*4)
		pd:finish()
		local ps = mainState:addState(PauseState(self))
		self:hideBeats()
		self:call(function(v)
			if v.setPaused then v:setPaused() end
		end)
		if self.currentWall then self.currentWall:setPaused() end
	end
	self.playOverlay.restartBg.onPress = function()
		self:fullRestart()
	end
end
function SpaceHero3:hideBeats()
	self.beats:call(function(v)
		if v.alphaTween then Tweener:removeTween(v.alphaTween) end
		v.alphaTween = Tweener:addTween(v.tint, {a=0})
	end)
end
function SpaceHero3:unPause()
	self:call(function(v)
		if v.setUnPause then v:setUnPause() end
	end)
	self.beats:call(function(v)
		if v.alphaTween then Tweener:removeTween(v.alphaTween) end
		v.tint.a = 255
	end)
	self.paused = false
	player:startTimer()
	if self.currentWall then self.currentWall:setUnPause() end
end
function SpaceHero3:newToggle(x, y, w, h)
	local toggleButton = DragArea(x, y, w, h)
	toggleButton.sprite = sprites['ui']['white.png']
	toggleButton.drawType = 'stretched'
	toggleButton:setColor(120, 180, 160, 0)
	toggleButton.scrollFactor = Vec2(0,0)
	return toggleButton
end
function SpaceHero3:setupUnderBg()
	mainbg = {}
	if SynthData.background then
		bg:setBackground(SynthData.background.under_color[1], SynthData.background.under_color[2], SynthData.background.under_color[3])
		if player.currentSheet then render:removeSheet(player.currentSheet) end
		sheets["under_bg"], sprites["under_bg"] = spriteLoader('background', 'png')
		player.currentSheet = sheets["under_bg"]
		render:addAtStart(player.currentSheet)
		sheets["under_bg"]:setBlendMode(1)
		-- for each of the items in the background table, test the type and add
		for i,v in ipairs(SynthData.background) do
			if v.type == 'starfield' then
				self:setupStarField(v)
			else
				self:setupParallax(v)
			end
		end
	elseif BackgroundData then
		self.vectorBackground = self:add(VectorBackground())
	end
end
function SpaceHero3:setupParallax(item)
	local b = SHSpace(sheets['under_bg'])
	b:setSpaceSprite(sprites["under_bg"][item.img]);
	b:setColor(255, 255, 255, 255)
	b:setSpeed(item.rate or 1, 1)
	b:setLoopY(false)
	table.insert(mainbg, b)
end
function SpaceHero3:setupStarField(item)
	local starField = SHStarField(sheets["under_bg"], item.number or 20);
	if item.rot then
		starField:setRot(unpack(item.rot))
	end
	starField:setScreens(item.screens or 1)
	if item.pos then
		if type(item.pos) == 'table' then
			starField:setPosition(item.pos[1]*r_scale/2, item.pos[2]*r_scale/2)
		else
			starField:setPosition(item.pos*r_scale/2)
		end
	end
	if item.scale then
		if type(item.scale) == 'table' then
			starField:setScale(item.scale[1], item.scale[2])
		else
			starField:setScale(item.scale)
		end
	end
	if item.rate then
		if type(item.rate) == 'table' then
			starField:setRate(item.rate[1], item.rate[2])
		else
			starField:setRate(item.scale)
		end
	end
	if type(item.img) == "table" then
		for i,v in ipairs(item.img) do
			starField:addStarSprite(sprites["under_bg"][v])
		end
	else
		starField:addStarSprite(sprites["under_bg"][item.img])
	end
	starField:fillStarField()
	table.insert(mainbg,starField)
end

--[[

 .oooooo..o oooooo   oooo ooooo      ooo   .oooooo.   
d8P'    `Y8  `888.   .8'  `888b.     `8'  d8P'  `Y8b  
Y88bo.        `888. .8'    8 `88b.    8  888          
 `"Y8888o.     `888.8'     8   `88b.  8  888          
     `"Y88b     `888'      8     `88b.8  888          
oo     .d8P      888       8       `888  `88b    ooo  
8""88888P'      o888o     o8o        `8   `Y8bood8P'                                                       

--]]

function SpaceHero3:sync()
	if self.gameOver then
		return
	end
	if self.nextSyncTriggers then
		for i,v in ipairs(self.nextSyncTriggers) do
			pd:addTrigger(player:getSampleAndNote(v[1], v[2]))
		end
		self.nextSyncTriggers = {}
	end

	if self.invincibilityCount > 0 then
		self.invincibilityCount = self.invincibilityCount - 1
	else
		self.hero.flash = false
	end
	if not self.winText then
		self.levelPointCount = self.levelPointCount+1
	end
	self.forceLoopOnce = self.forceLoopOnce - 1
	-- tick down the ocunauts health, this is sorta something to play around with more
	if not self.winText and not self.noBurn then
		local startHealth = self.healthBar.value
		local newHealth = math.max(0,self.healthBar.value-game_settings['sync_health_loss'])
		self.healthBar:setValue(newHealth)
		-- if the players health drops to 0, then kill their bonus
		if newHealth <= 0 and startHealth > 0 then
			self.scoreMultiplier = 1
			self.scoreboard:setMultiplier(1)
	    end
	end
	-- setup some containers to hold everything
	-- check to see if we need to add anything
	if not self.currentLoop then self.currentLoop = 1 end
	local levelStep = self.beatCount%16+1
	-- check to see if we have already cleared the whole loop
	local levelCleared = true
	for k, step in pairs(self.levelData[self.currentLoop]) do
		for k2, item in pairs(step) do
			local s = SynthData.getId(item[1])
			levelCleared = (item.hit or s.killer or item[3]==0) and levelCleared
		end
	end
	if self.noClear then
		levelCleared = false
	end
	if levelCleared and self.forceLoopOnce <= 0 then
		self:loopClear()
	end
	self.hero:setShotCharge(self.hero:getShotCharge()+game_settings['bullets_per_sync'])
	self.shotCharge:setValue(self.shotCharge.value+game_settings['bullets_per_sync'])
	self.backgroundLoop:setStep(self.beatCount)
	if self.triggerOld then self:triggerOld(levelStep) end
	local loopData = self.levelData[self.currentLoop][levelStep]
	self:triggerStep(loopData)
	self:moveExisting()
	self.beatCount = self.beatCount+1
end
function SpaceHero3:loopClear()
	-- up the health a little bit
	local newHealth = math.min(game_settings['max_health'], self.healthBar.value+game_settings['new_loop_health_gain'])
	self.healthBar:setValue(newHealth)
	self.currentLoop = self.currentLoop + 1
	self.forceLoopOnce = 16
	self.totalCoinCount = self.totalCoinCount+self.loopCoinCount
	self.loopCoinCount = 0
	if self.currentLoop > #self.levelData then
		self.currentLoop = #self.levelData
		if not self.winText then
			if not self.score then
				self.scoreboard:setScore(0)
				self.score = 0
			end
			-- should remove the pause button / etc
			self:remove(self.playOverlay)
			self.winText = self:add(WinOverlay(self.score, self.parseObject, self.songData, self))
		end
	end
	-- move the wall off 
	if self.currentWall then
		self.currentWall.velocity.x = -(bludG.camera.w)/beatsToTime(16)
		self.currentWall:explode()
	end
	self.currentWall = nil
	if self:currentLoopHasKeys() and not self.winText then
		self.currentWall = self.walls:addRecycle(Wall())
	end
	addTextPop("Loop Clear")
	-- move out background loops that are too old
	-- turn off the existing triggers so that they don't play now that we have cleared the step
	for i,v in ipairs(self.beats.members) do
		if v.alive and not v.no_sync then
			v.off = true
		end
	end
	self.backgroundLoop:emptyOld(self.currentLoop)
	self.hero:setShotCharge(self.hero:getShotCharge()+game_settings['bullets_per_loop'])
    self.shotCharge:setValue(self.shotCharge.value+game_settings['bullets_per_loop'])
end
function SpaceHero3:currentLoopHasKeys()
	for k, step in pairs(self.levelData[self.currentLoop]) do
		for k2, item in pairs(step) do
			if item[3]==1 then
				return true
			end
		end
	end
	return false
end
function SpaceHero3:screenHeightToNoteHeight(yPos)
	local playArea = self.ground.pos.y-(self.ceiling.pos.y+self.ceiling.h)
	yPos = yPos-(self.ceiling.pos.y+self.ceiling.h)
	return math.floor((yPos/playArea)*7)
end
function SpaceHero3:noteHeightToScreenHeight(note)
	-- this first calucalation could be cached
	-- in fact this entire thing could be memoized
    local playHeight = self.ground.pos.y-(self.ceiling.pos.y+self.ceiling.h)
    playHeight = playHeight/7
	return playHeight*note+(self.ceiling.pos.y+self.ceiling.h)
end
function SpaceHero3:getLoopCompletionPercentage()
	local keysHit = 0
	local keysTotal = 0
	for k, step in pairs(self.levelData[self.currentLoop]) do
		for k2, item in pairs(step) do
			local s = SynthData.getId(item[1])
			if item[3] ~= 0 and not s.killer then
				keysTotal = keysTotal + 1
				if item.hit then
					keysHit = keysHit + 1
				end
			end
		end
	end
	return keysHit/keysTotal
end
function SpaceHero3:triggerStep(loopData)
  if loopData then
    for k,v in pairs(loopData) do
      local s = SynthData.getId(loopData[k][1])
      -- build the new object with this type / synth
      -- before building, check to see if we have already put this character on screen...
      -- only for the characters that stay on screen until the end of the loop
      -- the 4th index is just a holder in the data for the entity. Kinda confusing, probably shouldn't do this, etc
      -- the last bit here make is so that things that have been hit don't get spawned
      if not loopData[k][4] or not s.single and not loopData[k].hit then
        local playHeight = self.ground.pos.y-(self.ceiling.pos.y+self.ceiling.h)
        playHeight = playHeight/7
        -- attempt to get a crosser, and recycle it
        local crosser = self.beats:recycleHard()
        if crosser then
        	crosser:recycle(bludG.camera.w+(self.hero.pos.x+self.hero.w),self:noteHeightToScreenHeight(k),s)
        else
        	crosser = self.beats:addRecycle(SynthObject(bludG.camera.w+(self.hero.pos.x+self.hero.w),self:noteHeightToScreenHeight(k),s))
        end
        crosser.h = playHeight
        loopData[k][4] = crosser
        crosser.s = s
        crosser.levelInfo = loopData[k]
        crosser.drawType = 'stretched'
        crosser.moves = true
        if not crosser.no_sync then
        	crosser.velocity.x = -(bludG.camera.w)/beatsToTime(16)
		end
        if crosser.levelInfo.hit then
          crosser:setColor(crosser.tint.r,crosser.tint.g,crosser.tint.b, 100)
        else
          crosser:setColor(crosser.tint.r,crosser.tint.g,crosser.tint.b, 255)
        end
        crosser.immovable = false
        crosser.count = 17
        if loopData[k][3]==1 then
          crosser:setKey(true)
        end
        crosser:setupComponents()
        if self.WinOverlay then
        	crosser:setColor(0,0,0,0)
        end
        -- if the object is a killer, then play its sound immediately
        if s.killer then
          if s.sound then
            pd:addTrigger(player:getSampleAndNote(s, 7-crosser.levelInfo[2]))
          end
        end
      end
    end
  end
end
function SpaceHero3:moveExisting()
  if not self.pulsers then self.pulsers = self:add(Pulsers()) end
  lastTime = bludG.last_t
  for i,v in ipairs(self.beats.members) do
    if v.sync then v:sync() end
    if v.alive and not v.no_sync then
      -- we are just going to lerp across, because that will give us better timing accuracy
      if v.moveTween then
        v.moveTween.onComplete(v);
        Tweener:removeTween(v.moveTween)
      end
      v.count = v.count - 1
      if v.alive then
        if v.pos.x+v.w <= 0 then
          v:kill()
        end         
      end
    end
  end
end

function SpaceHero3:triggerOld(levelStep)
	-- trigger the older patterns
	for i=self.currentLoop-3,self.currentLoop do
	    if self.levelData[i] and self.levelData[i][levelStep] then
	      for k,v in pairs(self.levelData[i][levelStep]) do
		    local s = SynthData.getId(v[1])
      		if v.hit then
      			if s.sound then
      				pd:addTrigger(player:getSampleAndNote(s, 7-v[2]))
				end
			end
	      end
	  	end
  	end
end
function SpaceHero3:resetLoop()

	Tweener:removeTween(self.bgTween)
	local targetBg;
	if self.bg_targetscrollx then
		targetBg = self.bg_targetscrollx+bludG.camera.w/16	
	else
		targetBg = self.bg_scrollx+bludG.camera.w/16
	end
	-- clear the hit values on everything, then reset to basic
	if not self.currentLoop then self.currentLoop = 1 end
	-- local levelStep = self.beatCount%16+1
	-- check to see if we have already cleared the whole loop
	for k, step in pairs(self.levelData[self.currentLoop]) do
		for k2, item in pairs(step) do
			self.levelData[self.currentLoop][k][k2][4] = nil
			item.hit = false
		end
	end 
	local oldBeats = self.beats
	-- convert all the existing beats to pulses
	for i,v in ipairs(oldBeats.members) do
		if v.alive then
			self:removeObject(v, oldBeats, true)
		end
	end
	-- then empty out all the beats
	self.beats = self:add(Group())
	self.beatCount = 0
end
function SpaceHero3:removeObject(v, oldBeats, removeGroup)
	Tweener:removeTween(v.moveTween)
	v.acceleration = Vec2(0, -200*r_scale-math.random()*50)
	Tweener:addTween(v.tint, {a=v.tint.a}, {time=math.random(), onComplete=function()
		v.moves = true
		v.fadeTween = Tweener:addTween(v.tint, {a=0}, {time=0.5+math.random(), onComplete=function()
			v:kill()
			-- check to see if they are all dead, and if so, remove this from the space hero thing
			if removeGroup and oldBeats:countLiving() <= 0 then
				self:remove(oldBeats)
			end
		end})
	end})
end

--[[
                             .o8                .             
                            "888              .o8             
oooo  oooo  oo.ooooo.   .oooo888   .oooo.   .o888oo  .ooooo.  
`888  `888   888' `88b d88' `888  `P  )88b    888   d88' `88b 
 888   888   888   888 888   888   .oP"888    888   888ooo888 
 888   888   888   888 888   888  d8(  888    888 . 888    .o 
 `V88V"V8P'  888bod8P' `Y8bod88P" `Y888""8o   "888" `Y8bod8P' 
             888                                              
            o888o                                             
]]
function SpaceHero3:update()
	if self.paused then return end
	self.bg_scrollx = self.bg_scrollx+((bludG.camera.w-(self.hero.pos.x+self.hero.w))/beatsToTime(16))*bludG.elapsed
	self.hero.acceleration:set(self.gravity)
	if self.jetOn or self.hero.fingerDown then
		self.hero.acceleration:add(self.hero.jetacceleration)
	end
	Group.update(self)
	self.hero.pos.y = math.max(self.ceiling.pos.y+self.ceiling.h, math.min(self.ground.pos.y, self.hero.pos.y))
	if self.hero.pos.y == self.ground.pos.y or self.hero.pos.y == self.ceiling.pos.y+self.ceiling.h then
		self.hero.velocity.y = 0
	end
	local killall = false
	if self.beats then
		self.beats:overlap(self.hero, function(a, b)
			self:handleBeatHit(a, b)
		end)
		self:handleShieldHit()
		self.beats:overlap(self.bullets, function(a, b)
			if b.alive and a.s and a.s.killer and not self.winText then
				pd:startList("fromOF")
				pd:addSymbol("enemyDestroy")
				pd:addFloat(a.pos.y/bludG.camera.h)
				pd:finish()
				addSmallTorus(b.pos)
				a:kill()
				b:kill()
			end     
		end)
		self.beats:call(function(v)
			if not v.no_ground_collide then
				v.pos.y = math.min(self.ground.pos.y, v.pos.y)
			end
		end)
	end
	if self.nonBeat then
		self.nonBeat:overlap(self.hero, function(a,b)
			killall = true
		end)
	end
	if killall then self:resetLoop() end
end
function SpaceHero3:handleShieldHit()
	if self.hero:hasShield() then
		self.beats:overlap(self.hero.shield, function(a, b)
			if a.s and a.s.killer and not self.winText then
				pd:startList("fromOF")
				pd:addSymbol("enemyDestroy")
				pd:addFloat(a.pos.y/bludG.camera.h)
				pd:finish()
				addSmallTorus(a.pos)
				-- attempt to blow away the current thing
				local e = self.deadEnemies:recycleHard()
				if e then
					e:recycle(a)
				else
					e = self.deadEnemies:addRecycle(DeadEnemy(a))
				end
				e.velocity = b.pos:copy():sub(a.pos):normalize():mult(-300*r_scale)
				e.rot = math.deg(e.velocity:copy():normalize():angle())
				a:kill()
			end
		end)      
	end
end
function SpaceHero3:handleBeatHit(a, b)
  if a.s and a.s.killer and not self.winText and self.invincibilityCount <= 0 then
    pd:startList("fromOF")
    pd:addSymbol("enemyHit")
    pd:addFloat(0)
    pd:finish()
    local newHealth = 0
    self.healthBar:setValue(newHealth)
    self.scoreboard:setMultiplier(1)
    self.scoreMultiplier = 1
    addTextPop("Bonus Down!")
    self.invincibilityCount = 4
    self.hero.flash = true
  elseif a.levelInfo then
    if not a.levelInfo.hit and not a.s.killer then
    	self.backgroundLoop:addItem(a.s, self.beatCount, a.pos.y, self.currentLoop)
		a.levelInfo.hit = true
		self.hero.pushPower = self.hero.pushPower+10;
		-- should loop through all of the beats and make sure the ones that are the same get turned off
		pd:addTrigger(player:getSampleAndNote(a.s, 7-a.levelInfo[2]))
		self.beats:call(function(beat)
			if beat.levelInfo.hit == true then
				beat:kill()
				beat:setColor(beat.tint.r, beat.tint.g, beat.tint.b, 100)
			end
		end)
		self.loopCoinCount = self.loopCoinCount+1
		if a.key then
			local newHealth = math.min(game_settings['max_health'], self.healthBar.value+game_settings['key_coin_health_gain'])
			if not self.scoreMultiplier then self.scoreMultiplier = 1 end
			if not self.score then self.score = 0 end
			self.score = self.score+1*self.scoreMultiplier
			self.scoreboard:setScore(self.score)
			self.healthBar:setValue(newHealth)
			local lineA = a:center():copy()
			local lineB = a:center():copy()
			if self.currentWall then
				self.currentWall.health.percent = 1-self:getLoopCompletionPercentage()
				Tweener:addTween(lineA, {x=self.currentWall.pos.x, y=self.currentWall.pos.y}, {time=0.4})
				Tweener:addTween(lineB, {x=self.currentWall.pos.x, y=self.currentWall.pos.y}, {time=0.3, onComplete=function()
					mainState.particles:addRecycle(ParticleHandler(lineB.x, lineB.y, "particle_data_heal.lua", sprites['ui']['circle.png'], {preroll=0.2, lifetime=0}))
				end})
				local line = mainState.particles:addRecycle(PlayerLine(lineA, lineB, true))
				local t = SynthData.key_tint
				line:setColor(t[1], t[2], t[3], 200)
				line.w = 30*r_scale
				Tweener:addTween(line, {w=4*r_scale}, {func='outExpo', time=0.3})
			end
		else
			if not self.scoreMultiplier then self.scoreMultiplier = 0 end
			self.scoreMultiplier = self.scoreMultiplier+1
			self.scoreboard:setMultiplier(self.scoreMultiplier)
			local newHealth = math.min(game_settings['max_health'], self.healthBar.value+game_settings['non_key_coin_health_gain'])
			self.healthBar:setValue(newHealth)
			addTorus(self.hero.pos)
		end
    end
  end
end
function SpaceHero3:fireBullet()
	if self.shotCharge.value > game_settings['shot_cost'] then
		self.hero:useShield()
		self.shotCharge:setValue(self.shotCharge.value - game_settings['shot_cost'])
		self.hero:setShotCharge(self.hero:getShotCharge()-game_settings['shot_cost'])
		pd:startList("fromOF")
		pd:addSymbol("fireShot")
		pd:addFloat(self.hero.rot)
		pd:addFloat(self.hero.pos.y/bludG.camera.h)
		pd:finish()
	else
		self.hero:useShieldDischarged()
		playEffectNew('shieldDischarge')
	end
end
function SpaceHero3:addToggle(x, y, w, h, fn, upfn)
	local o = self
	return toggleButton
end
function SpaceHero3:keyPressed(key)
	-- z: 59 x: 113
	if key == 59 or key == 122 then
		self.jetOn = true
	end
	if key == 113 or key == 120 then
		self:fireBullet()
	end
end
function SpaceHero3:keyReleased(key)
	if key == 59 or key == 122 then
		self.jetOn = false
	end
	if key == 39 then
		-- this should quit and restart the game
		self:fullRestart()
	end
end
function SpaceHero3:fullRestart()
	local sh3 = SpaceHero3(self.songData, self.parseObject, self.parent)
	mainState = WrapperState(Group());
	bludG.camera:zoomTo(0,0)
	mainState:changeState(sh3)
end
function addTorus(pos)
	local to = mainState.particles:addRecycle(TObject(pos.x, pos.y, {resolution=20})) 
	if SynthData.coin_tint then
		local t = SynthData.coin_tint
		to:setColor(t[1], t[2], t[3], 200)
	end
	Tweener:addTween(to.pos, {x=pos.x-100*r_scale}, {func="easeNone", time=1})
	Tweener:addTween(to.tint, {a=0, r=0, g=0, b=0}, {time=1.8, onComplete=function()
		to:kill()
	end})
	Tweener:addTween(to, {innerSize=80*r_scale, outerSize=90*r_scale}, {func="easeNone", time=.4, onComplete=function()
	end})
end
function addSmallTorus(pos)
	local to = mainState.particles:addRecycle(TObject(pos.x, pos.y, {resolution=20})) 
	if SynthData.coin_tint then
		local t = SynthData.coin_tint
		to:setColor(t[1], t[2], t[3], 200)
	end
	Tweener:addTween(to.pos, {x=pos.x-100*r_scale}, {func="easeNone", time=1})
	Tweener:addTween(to.tint, {a=0, r=0, g=0, b=0}, {time=1.8, onComplete=function()
		to:kill()
	end})
	Tweener:addTween(to, {innerSize=30*r_scale, outerSize=40*r_scale}, {func="easeNone", time=.4, onComplete=function()
	end})
end
function addTextPop(text)
	-- local text = mainState.particles:addRecycle(TextPop(bludG.camera.w/2, bludG.camera.h/2, text))
	-- text.text:setColor(255, 255, 255)
end