TurnShotLocalState = class(Group, function(o, match)
	bg:setBackground(getColor())
	Group.init(o)
	o.bullets = o:add(Group())
	o.players = o:add(Group())
	o.p1 = o.players:add(Player(bludG.camera.w/3, bludG.camera.h/2))
	o.p1:setColor(230, 100, 100)
	o.p2 = o.players:add(Player(bludG.camera.w/3*2, bludG.camera.h/2))
	o.p2:setColor(100, 100, 230)

	o.drag = o:add(DragArea(0,0,bludG.camera.w, bludG.camera.h))
	o.drag.onStart = function(da,x,y,id)
		o:dragStart(da, x, y, id)
	end
	o.drag.onMove = function(da,x,y,id)
		o:dragMove(da, x, y, id)
	end
	o.drag.onPress = function(da,x,y,id) 
		o:dragPress(da, x, y, id)
	end
	o.drag.scrollFactor = Vec2(0,0)

	o.moves = {}
	o.dieRoll = {}
	o.frameCount = 0
	o.replay = {}
	o.currentState = 'turns'

	o.currentPlayer = 1
	o.myPlayer = o.p2
	o.theirPlayer = o.p1

end)
function TurnShotLocalState:setupNetwork()
	self.networkSetup = true
	self.gkMatch = bludGKMatch()
	local parent = self
	self.gkMatch:setDataCallback(function(data)
		self:gotNetworkData(data)
	end)
	self.gkMatch:createMatch(function(success)
		if success then
			parent:startNetworkGame()
		else
			parent.networkSetup = nil
		end
	end)
end
function TurnShotLocalState:nextState()
	if self.currentState == 'turns' then
		if self.currentPlayer == 1 then
			self.currentPlayer = 2
		elseif self.currentPlayer == 2 then
			self.currentPlayer = 1
			self.currentState = 'play_turn'
			self.turnCount = 30
		end
	else
		self.currentState = 'turns'
	end
end
function TurnShotLocalState:dragStart(da, x, y, id)
	if self.previewLine then self:remove(self.previewLine) end
	local targetPlayer = self.myPlayer
	if self.currentPlayer == 2 then
		targetPlayer = self.theirPlayer
	end
	self.previewLine = self:add(PlayerLine(targetPlayer.pos, Vec2(x, y)))
	self.previewLine.w = 2
	self.previewLine:setColor(targetPlayer.tint.r, targetPlayer.tint.g, targetPlayer.tint.b)
end
function TurnShotLocalState:dragMove(da, x, y, id)
	local targetPlayer = self.myPlayer
	if self.currentPlayer == 2 then
		targetPlayer = self.theirPlayer
	end
	if not self.previewLine then self:dragStart(da, x, y, id) end
	self.previewLine.b = Vec2(x, y)
end
function TurnShotLocalState:dragPress(da, x, y, id)
	local targetPlayer = self.myPlayer
	if self.currentPlayer == 2 then
		targetPlayer = self.theirPlayer
	end
	if not self.previewLine then self:dragStart(da, x, y, id) end
	self:setMove(targetPlayer, Vec2(x, y))
	self:nextState()
end
function TurnShotLocalState:setMove(targetPlayer, movement)
	self.moves[targetPlayer] = movement
	if self.moves[self.myPlayer] and self.moves[self.theirPlayer] then
		self:commitMoves()
	end
end
function TurnShotLocalState:commitMoves()
	if self.moves[self.p1] then
		local p1v = self.moves[self.p1]:copy():sub(self.p1.pos)
		self.p1.vel = self.p1.vel:add(p1v:copy():normalize():mult(-1))
		-- add a bullet
		local b = self.bullets:add(Bullet(self.p1.pos.x, self.p1.pos.y))
		b.vel = p1v:normalize():mult(3):add(self.p1.vel)
	end
	if self.moves[self.p2] then
		local p2v = self.moves[self.p2]:copy():sub(self.p2.pos)
		self.p2.vel = self.p2.vel:add(p2v:copy():normalize():mult(-1))
		local b = self.bullets:add(Bullet(self.p2.pos.x, self.p2.pos.y))
		b.vel = p2v:normalize():mult(3):add(self.p2.vel)
	end
	if self.replay then
		self.replay[self.frameCount] = {self.moves[self.p1], self.moves[self.p2]}
	end
	self.moves = {}
end
function loopToScreen(o)
	if o.pos.x < 0 then
		o.pos.x = bludG.camera.w
	end
	if o.pos.x > bludG.camera.w then
		o.pos.x = 0
	end
	if o.pos.y < 0 then
		o.pos.y = bludG.camera.h
	end
	if o.pos.y > bludG.camera.h then
		o.pos.y = 0
	end
end
function bounceToScreen(o)
	if o.pos.x < 0 then
		o.pos.x = 0
		o.vel.x = o.vel.x*-1
	end
	if o.pos.x > bludG.camera.w then
		o.pos.x = bludG.camera.w
		o.vel.x = o.vel.x*-1
	end
	if o.pos.y < 0 then
		o.pos.y = 0
		o.vel.y = o.vel.y*-1
	end
	if o.pos.y > bludG.camera.h then
		o.pos.y = bludG.camera.h
		o.vel.y = o.vel.y*-1
	end
end
function TurnShotLocalState:update()
	if self.currentState == 'play_turn' then
		self.frameCount = self.frameCount + 1
		-- step all of the bullets and stuff forward
		self.players:call(function(p)
			p.pos:add(p.vel)
			p.vel:mult(0.99)
			bounceToScreen(p)
		end)
		self.bullets:call(function(p)
			p.pos:add(p.vel)
			-- bounceToScreen(p)
			p.age = p.age+1
		end)
		self.players:overlap(self.bullets, function(a, b)
			if b.age > 15 then
				if a == self.p1 then
					if not self.gameOver then
						self.gameoverstate = self:add(TurnShotGameOver(self, "blue wins"))
					end
				end
				if a == self.p2 then
					if not self.gameOver then
						self.gameoverstate = self:add(TurnShotGameOver(self,  "red wins"))
					end
				end
			end
		end)
		self.turnCount = self.turnCount - 1
		if self.turnCount < 0 then
			self:nextState()
		end
	end
	if self.gameoverstate then
		self.gameoverstate:update()
	end
end