-- TSerial v1.23, a simple table serializer which turns tables into Lua script
-- by Taehl (SelfMadeSpirit@gmail.com)

-- Usage: table = TSerial.unpack( TSerial.pack(table) )
TSerial = {}
function TSerial.pack(t)
	assert(type(t) == "table", "Can only TSerial.pack tables.")
	local s = "{"
	for k, v in pairs(t) do
		local tk, tv = type(k), type(v)
		if tk == "boolean" then k = k and "[true]" or "[false]"
		elseif tk == "string" then if string.find(k, "[%c%p%s]") then k = '["'..k..'"]' end
		elseif tk == "number" then k = "["..k.."]"
		elseif tk == "table" then k = "["..TSerial.pack(k).."]"
		else error("Attempted to Tserialize a table with an invalid key: "..tostring(k))
		end
		if tv == "boolean" then v = v and "true" or "false"
		elseif tv == "string" then v = string.format("%q", v)
		elseif tv == "number" then	-- no change needed
		elseif tv == "table" then v = TSerial.pack(v)
		else error("Attempted to Tserialize a table with an invalid value: "..tostring(v))
		end
		s = s..k.."="..v..","
	end
	return s.."}"
end

function TSerial.unpack(s)
	assert(type(s) == "string", "Can only TSerial.unpack strings.")
	assert(loadstring("TSerial.table="..s))()
	local t = TSerial.table
	TSerial.table = nil
	return t
end

Bullet = class(CenteredObject, function(o, x, y)
	CenteredObject.init(o, x, y, 5*r_scale, 5*r_scale)
	o.sprite = sprites['ui']['circle.png']
	o.scale = o.w/o.sprite:getWidth()
	o.vel = Vec2(0,0)
	o:setColor(0,0,0)
	o.age = 0
end)
Player = class(CenteredObject, function(o, x, y)
	CenteredObject.init(o, x, y, 15*r_scale, 15*r_scale)
	o.sprite = sprites['ui']['circle.png']
	o.scale = o.w/o.sprite:getWidth()
	o.vel = Vec2(0,0)
end)
TurnShotState = class(Group, function(o, match)
	bg:setBackground(getColor())
	Group.init(o)
	o.bullets = o:add(Group())
	o.players = o:add(Group())
	o.p1 = o.players:add(Player(bludG.camera.w/3, bludG.camera.h/2))
	o.p1:setColor(230, 100, 100)
	o.p2 = o.players:add(Player(bludG.camera.w/3*2, bludG.camera.h/2))
	o.p2:setColor(100, 100, 230)

	o.drag = o:add(DragArea(0,0,bludG.camera.w, bludG.camera.h))
	o.drag.onStart = function(da,x,y,id)
		o:dragStart(da, x, y, id)
	end
	o.drag.onMove = function(da,x,y,id)
		o:dragMove(da, x, y, id)
	end
	o.drag.onPress = function(da,x,y,id) 
		o:dragPress(da, x, y, id)
	end
	o.drag.scrollFactor = Vec2(0,0)
	o.currentState = 'network_start'
	if match then
		o.networkSetup = true
		o.gkMatch = match
		o:startNetworkGame()
		o.gkMatch:setDataCallback(function(data)
			o:gotNetworkData(data)
		end)
	end
	o.moves = {}
	o.dieRoll = {}
	o.frameCount = 0
	o.replay = {}
end)
function TurnShotState:setupNetwork()
	self.networkSetup = true
	self.gkMatch = bludGKMatch()
	local parent = self
	self.gkMatch:setDataCallback(function(data)
		self:gotNetworkData(data)
	end)
	self.gkMatch:createMatch(function(success)
		if success then
			parent:startNetworkGame()
		else
			parent.networkSetup = nil
		end
	end)
end
function TurnShotState:startNetworkGame()
	self.currentState = 'waitForReady'
	self.waitCount = 30
end
function TurnShotState:gotNetworkData(data)
	local data = TSerial.unpack(data)
	if type(data) == 'table' and data.type then
		if data.type == 'readyToRoll' then
			if self.currentState == 'waitForReady' then
				local moveData = {
					['type'] = 'readyToRoll';
				}
				self.gkMatch:sendData(TSerial.pack(moveData))
				self.currentState = 'waitForRoll'
				-- roll a die and select our player
				local dieRoll = {
					['type'] = 'dieRoll';
					['count'] = math.random();
				}
				self:gotDieRoll("mine", dieRoll.count)
				-- send the data
				self.gkMatch:sendData(TSerial.pack(dieRoll))
			end
		elseif data.type == 'dieRoll' then
			self:gotDieRoll('theirs', data.count)
		elseif data.type == 'turnData' then
			self:setMove(self.theirPlayer, Vec2(data.move[1], data.move[2]))
		end
	end
end
function TurnShotState:gotDieRoll(whom, count)
	self.dieRoll[whom] = count
	if self.dieRoll.mine and self.dieRoll.theirs then
		self.currentState = 'submit_move'
		if self.dieRoll.theirs > self.dieRoll.mine then
			self.myPlayer = self.p2
			self.theirPlayer = self.p1
		else
			self.myPlayer = self.p1
			self.theirPlayer = self.p2
		end
		-- should probably put in a reroll condition
	end
end
function TurnShotState:nextState()
	if self.currentState == 'submit_move' then
		self.currentState = 'wait_for_move'
	elseif self.currentState ==  'wait_for_move' then
		self.currentState = 'play_turn'
		self.turnCount = 30
	elseif self.currentState ==  'play_turn' then
		self.currentState = 'submit_move'
	end
end
function TurnShotState:dragStart(da, x, y, id)
	if not self.networkSetup then
		self:setupNetwork()
		return nil
	end
	if self.previewLine then self:remove(self.previewLine) end
	local targetPlayer = self.myPlayer
	if self.currentState == 'submit_move' then
		self.previewLine = self:add(PlayerLine(targetPlayer.pos, Vec2(x, y)))
		self.previewLine.w = 2
		self.previewLine:setColor(targetPlayer.tint.r, targetPlayer.tint.g, targetPlayer.tint.b)
	end
end
function TurnShotState:dragMove(da, x, y, id)
	if not self.networkSetup then return nil end
	local targetPlayer = nil
	local targetPlayer = self.myPlayer
	if self.currentState == 'submit_move' then
		if not self.previewLine then self:dragStart(da, x, y, id) end
		self.previewLine.b = Vec2(x, y)
	end
end
function TurnShotState:dragPress(da, x, y, id)
	if not self.networkSetup then return nil end
	local targetPlayer = nil
	local targetPlayer = self.myPlayer
	if self.currentState == 'submit_move' then
		if not self.previewLine then self:dragStart(da, x, y, id) end
		self:setMove(targetPlayer, Vec2(x, y))
		self:nextState()
	end
end
function TurnShotState:setMove(targetPlayer, movement)
	self.moves[targetPlayer] = movement
	if targetPlayer == self.myPlayer then
		local moveData = {
			['type'] = 'turnData';
			['move'] = {movement.x, movement.y};
		}
		self.gkMatch:sendData(TSerial.pack(moveData))
	end
	if self.moves[self.myPlayer] and self.moves[self.theirPlayer] then
		self:commitMoves()
		self:nextState()
	end
end
function TurnShotState:commitMoves()
	if self.moves[self.p1] then
		local p1v = self.moves[self.p1]:copy():sub(self.p1.pos)
		self.p1.vel = self.p1.vel:add(p1v:copy():normalize():mult(-1))
		-- add a bullet
		local b = self.bullets:add(Bullet(self.p1.pos.x, self.p1.pos.y))
		b.vel = p1v:normalize():mult(3):add(self.p1.vel)
	end
	if self.moves[self.p2] then
		local p2v = self.moves[self.p2]:copy():sub(self.p2.pos)
		self.p2.vel = self.p2.vel:add(p2v:copy():normalize():mult(-1))
		local b = self.bullets:add(Bullet(self.p2.pos.x, self.p2.pos.y))
		b.vel = p2v:normalize():mult(3):add(self.p2.vel)
	end
	if self.replay then
		self.replay[self.frameCount] = {self.moves[self.p1], self.moves[self.p2]}
	end
	self.moves = {}
end
function loopToScreen(o)
	if o.pos.x < 0 then
		o.pos.x = bludG.camera.w
	end
	if o.pos.x > bludG.camera.w then
		o.pos.x = 0
	end
	if o.pos.y < 0 then
		o.pos.y = bludG.camera.h
	end
	if o.pos.y > bludG.camera.h then
		o.pos.y = 0
	end
end
function bounceToScreen(o)
	if o.pos.x < 0 then
		o.pos.x = 0
		o.vel.x = o.vel.x*-1
	end
	if o.pos.x > bludG.camera.w then
		o.pos.x = bludG.camera.w
		o.vel.x = o.vel.x*-1
	end
	if o.pos.y < 0 then
		o.pos.y = 0
		o.vel.y = o.vel.y*-1
	end
	if o.pos.y > bludG.camera.h then
		o.pos.y = bludG.camera.h
		o.vel.y = o.vel.y*-1
	end
end

function TurnShotState:update()
	if self.currentState == 'waitForReady' then
		self.waitCount = self.waitCount - 1
		if self.waitCount < 0 then
			local moveData = {
				['type'] = 'readyToRoll';
			}
			self.gkMatch:sendData(TSerial.pack(moveData))
			self.waitCount = 30
		end
	end
	if self.currentState == 'play_turn' then
		self.frameCount = self.frameCount + 1
		-- step all of the bullets and stuff forward
		self.players:call(function(p)
			p.pos:add(p.vel)
			p.vel:mult(0.99)
			bounceToScreen(p)
		end)
		self.bullets:call(function(p)
			p.pos:add(p.vel)
			-- bounceToScreen(p)
			p.age = p.age+1
		end)
		self.players:overlap(self.bullets, function(a, b)
			if b.age > 15 then
				if a == self.p1 then
					if not self.gameOver then
						self.gameoverstate = self:add(TurnShotGameOver(self, "blue wins"))
					end
				end
				if a == self.p2 then
					if not self.gameOver then
						self.gameoverstate = self:add(TurnShotGameOver(self,  "red wins"))
					end
				end
			end
		end)
		self.turnCount = self.turnCount - 1
		if self.turnCount < 0 then
			self:nextState()
		end
	end
	if self.gameoverstate then
		self.gameoverstate:update()
	end
end