ScrollingBg = class(Group, function(o, name, data)
  Group.init(o)
  -- if we are on ipad, then account for the ratio difference
  local screenRatio = 320/480
  local height = bludG.camera.h
  local heightDiff = 0
  local scaleUp = 1
  if ipad == "_ip5" then
    screenRatio = 640/1136
  end
  if ipad ~= "" then
    height = bludG.camera.w*screenRatio
    heightDiff = bludG.camera.h-640*(r_scale*0.5)
  end
  -- load in the data and set the defaults
  o.bgElements = o:add(Group())
  o.svgs = {}
  if type(name)=='string' then
      local s = SvgLoadFromPF("/" .. player.currentWorld .. "/backgroundSvg/" .. name .. ".svg", true, nil, false)
      table.insert(o.svgs, s)
  else
    for i,v in ipairs(name) do
      local s = SvgLoadFromPF("/" .. player.currentWorld .. "/backgroundSvg/" .. v .. ".svg", true, nil, false)
      table.insert(o.svgs, s)
    end
  end
  local b = o.bgElements:add(Object(math.random()*bludG.camera.w,heightDiff/2,10,10))
  b.svg = o.svgs[math.random(1, #o.svgs)]
  b.drawType = 'svg'
  local density = data.density or 1
  local w = 0
  for i,v in ipairs(o.svgs) do
    w = math.max(w, v:getMaxX()*density)
  end
  b.w = w
  if data.tint then
    b:setColor(data.tint[1], data.tint[2], data.tint[3])
  end
  for i=1,math.ceil(bludG.camera.w/w)+1 do
    local l = o.bgElements:add(Object(i*w+b.pos.x,heightDiff/2,10,10))
    l.drawType = 'svg'
    l.svg = o.svgs[math.random(1, #o.svgs)]
    l:setColor(b.tint.r, b.tint.g, b.tint.b)
    l.w = w
    o.farRight = l
  end
  o.camera = 0
  o.rate = data.rate or 1
end)
function ScrollingBg:getCount()
  local c = 0
  self.bgElements:call(function(v)
    c = c+v.svg:getCount()
  end)
  return c
end
function ScrollingBg:update()
  -- samt = math.max(1, samt*0.9)
  local camPos = 0 
  if mainState.substate.bg_scrollx then
    camPos = -mainState.substate.bg_scrollx
  end
  self.camera = camPos*self.rate
  local c = 0
  -- color cycle bug
  -- local r,g,b = HSVtoRGB(bludG.last_t/10+c*0.2+0.5, 1, 1)
  -- bg:setBackground(r,g,b)

  self.bgElements:call(function(v)
    local r,g,b = HSVtoRGB(bludG.last_t/5+c*0.2, 1, 1)
    v.pos.x = (self.camera+v.w*c)%(bludG.camera.w+v.w*2)-v.w
    -- swap in a new svg if the current one is fully off the screen
    if v.pos.x > bludG.camera.w+50*r_scale then
      -- color cycle bug
    -- v:setColor(r,g,b)
      v.svg = self.svgs[math.random(1, #self.svgs)]
    end
    c = c+1
  end)
  Group.update(self)
end

function ScrollingBg:draw()
  if not self.rotSpeed then self.rotSpeed = math.random()*0.8+0.1 end
  if not self.rotAmt then self.rotAmt = math.random()*0.1+0.01 end

  self.bgElements:call(function(v)
    if not self.noRot then
      v.svg:resetTransform()
      v.svg:rotateAround(math.sin(bludG.last_t*self.rotSpeed+(v.pos.x/r_scale)*0.01)*self.rotAmt, v.svg:getMaxX()/2,v.svg:getMaxY())
    end
    v:draw()
  end)
end
VectorBackground = class(Group, function(o)
	bg:setBackground(BackgroundData.under_color[1], BackgroundData.under_color[2], BackgroundData.under_color[3])
	Group.init(o)
  if BackgroundData.gradient_bg then
    local c1 = BackgroundData.gradient_bg[1]
    local c2 = BackgroundData.gradient_bg[2]
    local gbg = o:add(GradientObject(0,0,bludG.camera.w,bludG.camera.h,bludColor(c1[1], c1[2], c1[3]), bludColor(c2[1], c2[2], c2[3])))
    gbg.sheet = sheets['bg']
    gbg.sprite = sprites['bg']['white.png']
  end
	local c = 0
	for i,v in ipairs(BackgroundData) do
		if v.svg then
			local s = o:add(ScrollingBg(v.svg,v))
      if v.noRot then s.noRot = true end
			c = c+s:getCount()
		end
	end
end)
function VectorBackground:update()
	Group.update(self)
	self.bg_scrollx = -bludG.last_t*100
end
-- function VectorBackground:draw()
--   -- body
-- end